import sys
import unittest
from builtins import object

from gaphas.state import reversible_pair, observed, _reverse


class SList(object):
    def __init__(self):
        self.list = list()

    def add(self, node, before=None):
        if before:
            self.list.insert(self.list.index(before), node)
        else:
            self.list.append(node)

    add = observed(add)

    @observed
    def remove(self, node):
        self.list.remove(self.list.index(node))


class StateTestCase(unittest.TestCase):
    def test_adding_pair(self):
        """Test adding reversible pair
        """
        reversible_pair(
            SList.add,
            SList.remove,
            bind1={"before": lambda self, node: self.list[self.list.index(node) + 1]},
        )

        if sys.version_info.major >= 3:  # Modern Python
            self.assertTrue(SList.add in _reverse)
            self.assertTrue(SList.remove in _reverse)
        else:  # Legacy Python
            self.assertTrue(SList.add.__func__ in _reverse)
            self.assertTrue(SList.remove.__func__ in _reverse)



"""
Some extra picklers needed to gracefully dump and load a canvas.
"""

from future import standard_library

standard_library.install_aliases()
import copyreg


# Allow instancemethod to be pickled:
import types


def construct_instancemethod(funcname, self, clazz):
    func = getattr(clazz, funcname)
    return types.MethodType(func, self, clazz)


def reduce_instancemethod(im):
    return (
        construct_instancemethod,
        (im.__func__.__name__, im.__self__, im.__self__.__class__),
    )


copyreg.pickle(types.MethodType, reduce_instancemethod, construct_instancemethod)


# Allow cairo.Matrix to be pickled:

import cairo


def construct_cairo_matrix(*args):
    return cairo.Matrix(*args)


def reduce_cairo_matrix(m):
    return construct_cairo_matrix, tuple(m)


copyreg.pickle(cairo.Matrix, reduce_cairo_matrix, construct_cairo_matrix)


# vim:sw=4:et:ai



"""
Test cases for the View class.
"""
from __future__ import division

import math
import unittest

from gi.repository import Gtk

from gaphas.canvas import Canvas
from gaphas.examples import Box
from gaphas.item import Line
from gaphas.view import View, GtkView


class ViewTestCase(unittest.TestCase):
    def test_bounding_box_calculations(self):
        """
        A view created before and after the canvas is populated should contain
        the same data.
        """
        canvas = Canvas()

        window1 = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        view1 = GtkView(canvas=canvas)
        window1.add(view1)
        view1.realize()
        window1.show_all()

        box = Box()
        box.matrix = (1.0, 0.0, 0.0, 1, 10, 10)
        canvas.add(box)

        line = Line()
        line.fyzzyness = 1
        line.handles()[1].pos = (30, 30)
        # line.split_segment(0, 3)
        line.matrix.translate(30, 60)
        canvas.add(line)

        window2 = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        view2 = GtkView(canvas=canvas)
        window2.add(view2)
        window2.show_all()

        # Process pending (expose) events, which cause the canvas to be drawn.
        while Gtk.events_pending():
            Gtk.main_iteration()

        try:
            assert view2.get_item_bounding_box(box)
            assert view1.get_item_bounding_box(box)
            assert view1.get_item_bounding_box(box) == view2.get_item_bounding_box(
                box
            ), (
                "%s != %s"
                % (view1.get_item_bounding_box(box), view2.get_item_bounding_box(box))
            )
            assert view1.get_item_bounding_box(line) == view2.get_item_bounding_box(
                line
            ), (
                "%s != %s"
                % (view1.get_item_bounding_box(line), view2.get_item_bounding_box(line))
            )
        finally:
            window1.destroy()
            window2.destroy()

    def test_get_item_at_point(self):
        """
        Hover tool only reacts on motion-notify events
        """
        canvas = Canvas()
        view = GtkView(canvas)
        window = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        window.add(view)
        window.show_all()

        box = Box()
        canvas.add(box)
        # No gtk main loop, so updates occur instantly
        assert not canvas.require_update()
        box.width = 50
        box.height = 50

        # Process pending (expose) events, which cause the canvas to be drawn.
        while Gtk.events_pending():
            Gtk.main_iteration()

        assert len(view._qtree._ids) == 1
        assert not view._qtree._bucket.bounds == (
            0,
            0,
            0,
            0,
        ), view._qtree._bucket.bounds

        assert view.get_item_at_point((10, 10)) is box
        assert view.get_item_at_point((60, 10)) is None

        window.destroy()

    def test_get_handle_at_point(self):
        canvas = Canvas()
        view = GtkView(canvas)
        window = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        window.add(view)
        window.show_all()

        box = Box()
        box.min_width = 20
        box.min_height = 30
        box.matrix.translate(20, 20)
        box.matrix.rotate(math.pi / 1.5)
        canvas.add(box)

        i, h = view.get_handle_at_point((20, 20))
        assert i is box
        assert h is box.handles()[0]

    def test_get_handle_at_point_at_pi_div_2(self):
        canvas = Canvas()
        view = GtkView(canvas)
        window = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        window.add(view)
        window.show_all()

        box = Box()
        box.min_width = 20
        box.min_height = 30
        box.matrix.translate(20, 20)
        box.matrix.rotate(math.pi / 2)
        canvas.add(box)

        p = canvas.get_matrix_i2c(box).transform_point(0, 20)
        p = canvas.get_matrix_c2i(box).transform_point(20, 20)
        i, h = view.get_handle_at_point((20, 20))
        assert i is box
        assert h is box.handles()[0]

    def test_item_removal(self):
        canvas = Canvas()
        view = GtkView(canvas)
        window = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        window.add(view)
        window.show_all()

        box = Box()
        canvas.add(box)
        # No gtk main loop, so updates occur instantly
        assert not canvas.require_update()

        # Process pending (expose) events, which cause the canvas to be drawn.
        while Gtk.events_pending():
            Gtk.main_iteration()

        assert len(canvas.get_all_items()) == len(view._qtree)

        view.focused_item = box
        canvas.remove(box)

        assert len(canvas.get_all_items()) == 0
        assert len(view._qtree) == 0

        window.destroy()

    def test_view_registration(self):
        canvas = Canvas()

        # Simple views do not register on the canvas

        view = View(canvas)
        assert len(canvas._registered_views) == 0

        box = Box()
        canvas.add(box)

        # By default no complex updating/calculations are done:
        assert view not in box._matrix_i2v
        assert view not in box._matrix_v2i

        # GTK view does register for updates though

        view = GtkView(canvas)
        assert len(canvas._registered_views) == 1

        # No entry, since GtkView is not realized and has no window
        assert view not in box._matrix_i2v
        assert view not in box._matrix_v2i

        window = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        window.add(view)
        window.show_all()

        # Now everything is realized and updated
        assert view in box._matrix_i2v
        assert view in box._matrix_v2i

        view.canvas = None
        assert len(canvas._registered_views) == 0

        assert view not in box._matrix_i2v
        assert view not in box._matrix_v2i

        view.canvas = canvas
        assert len(canvas._registered_views) == 1

        assert view in box._matrix_i2v
        assert view in box._matrix_v2i

    def test_view_registration_2(self):
        """
        Test view registration and destroy when view is destroyed.
        """
        canvas = Canvas()
        view = GtkView(canvas)
        window = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        window.add(view)
        window.show_all()

        box = Box()
        canvas.add(box)

        assert hasattr(box, "_matrix_i2v")
        assert hasattr(box, "_matrix_v2i")

        assert box._matrix_i2v[view]
        assert box._matrix_v2i[view]

        assert len(canvas._registered_views) == 1
        assert view in canvas._registered_views

        window.destroy()

        assert len(canvas._registered_views) == 0

        assert view not in box._matrix_i2v
        assert view not in box._matrix_v2i

    def test_scroll_adjustments_signal(self):
        sc = Gtk.ScrolledWindow()
        view = GtkView(Canvas())
        sc.add(view)

        assert view.hadjustment
        assert view.vadjustment
        assert view.hadjustment.get_value() == 0.0
        assert view.hadjustment.get_lower() == 0.0
        assert view.hadjustment.get_upper() == 1.0
        assert view.hadjustment.get_step_increment() == 0.0
        assert view.hadjustment.get_page_increment() == 1.0
        assert view.hadjustment.get_page_size() == 1.0
        assert view.vadjustment.get_value() == 0.0
        assert view.vadjustment.get_lower() == 0.0
        assert view.vadjustment.get_upper() == 1.0
        assert view.vadjustment.get_step_increment() == 0.0
        assert view.vadjustment.get_page_increment() == 1.0
        assert view.vadjustment.get_page_size() == 1.0

    def test_scroll_adjustments(self):
        sc = Gtk.ScrolledWindow()
        view = GtkView(Canvas())
        sc.add(view)
        assert sc.get_hadjustment() is view.hadjustment
        assert sc.get_vadjustment() is view.vadjustment


if __name__ == "__main__":
    unittest.main()

# vim:sw=4:et:ai



from __future__ import print_function

import pickle
import unittest
from builtins import object

from future import standard_library
from gi.repository import Gtk

from gaphas.canvas import Canvas
from gaphas.examples import Box
from gaphas.item import Element, Line
from gaphas.view import View, GtkView

# Ensure extra pickle reducers/reconstructors are loaded:
import gaphas.picklers

standard_library.install_aliases()


class MyPickler(pickle.Pickler):
    def save(self, obj):
        # print('saving obj', obj, type(obj))
        try:
            return pickle.Pickler.save(self, obj)
        except pickle.PicklingError as e:
            print("Error while pickling", obj, self.dispatch.get(type(obj)))
            raise e


class MyDisconnect(object):
    """
    Disconnect object should be located at top-level, so the pickle code
    can find it.
    """

    def __call__(self):
        pass


def create_canvas():
    canvas = Canvas()
    box = Box()
    canvas.add(box)
    box.matrix.translate(100, 50)
    box.matrix.rotate(50)
    box2 = Box()
    canvas.add(box2, parent=box)

    line = Line()
    line.handles()[0].visible = False
    line.handles()[0].connected_to = box
    line.handles()[0].disconnect = MyDisconnect()
    line.handles()[0].connection_data = 1

    canvas.add(line)

    canvas.update()

    return canvas


class PickleTestCase(unittest.TestCase):
    def test_pickle_element(self):
        item = Element()

        pickled = pickle.dumps(item)
        i2 = pickle.loads(pickled)

        assert i2
        assert len(i2.handles()) == 4

    def test_pickle_line(self):
        item = Line()

        pickled = pickle.dumps(item)
        i2 = pickle.loads(pickled)

        assert i2
        assert len(i2.handles()) == 2

    def test_pickle(self):
        canvas = create_canvas()

        pickled = pickle.dumps(canvas)
        c2 = pickle.loads(pickled)

        assert type(canvas._tree.nodes[0]) is Box
        assert type(canvas._tree.nodes[1]) is Box
        assert type(canvas._tree.nodes[2]) is Line

    def test_pickle_connect(self):
        """
        Persist a connection.
        """
        canvas = Canvas()
        box = Box()
        canvas.add(box)
        box2 = Box()
        canvas.add(box2, parent=box)

        line = Line()
        line.handles()[0].visible = False
        line.handles()[0].connected_to = box
        line.handles()[0].disconnect = MyDisconnect()
        line.handles()[0].connection_data = 1

        canvas.add(line)

        pickled = pickle.dumps(canvas)
        c2 = pickle.loads(pickled)

        assert type(canvas._tree.nodes[0]) is Box
        assert type(canvas._tree.nodes[1]) is Box
        assert type(canvas._tree.nodes[2]) is Line
        assert c2.solver

        line2 = c2._tree.nodes[2]
        h = line2.handles()[0]
        assert h.visible == False
        assert h.connected_to is c2._tree.nodes[0]

        # connection_data and disconnect have not been persisted
        assert h.connection_data == 1, h.connection_data
        assert h.disconnect, h.disconnect
        assert callable(h.disconnect)
        assert h.disconnect() is None, h.disconnect()

    def test_pickle_with_view(self):
        canvas = create_canvas()

        pickled = pickle.dumps(canvas)

        c2 = pickle.loads(pickled)

        view = View(canvas=c2)

        import cairo

        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
        cr = cairo.Context(surface)
        view.update_bounding_box(cr)
        cr.show_page()
        surface.flush()
        surface.finish()

    def test_pickle_with_gtk_view(self):
        canvas = create_canvas()

        pickled = pickle.dumps(canvas)

        c2 = pickle.loads(pickled)

        win = Gtk.Window()
        view = GtkView(canvas=c2)
        win.add(view)

        view.show()
        win.show()

        view.update()

    def test_pickle_with_gtk_view_with_connection(self):
        canvas = create_canvas()
        box = canvas._tree.nodes[0]
        assert isinstance(box, Box)
        line = canvas._tree.nodes[2]
        assert isinstance(line, Line)

        view = GtkView(canvas=canvas)

        #        from gaphas.tool import ConnectHandleTool
        #        handle_tool = ConnectHandleTool()
        #        handle_tool.connect(view, line, line.handles()[0], (40, 0))
        #        assert line.handles()[0].connected_to is box, line.handles()[0].connected_to
        #        assert line.handles()[0].connection_data
        #        assert line.handles()[0].disconnect
        #        assert isinstance(line.handles()[0].disconnect, object), line.handles()[0].disconnect

        import io

        f = io.BytesIO()
        pickler = MyPickler(f)
        pickler.dump(canvas)
        pickled = f.getvalue()

        c2 = pickle.loads(pickled)

        win = Gtk.Window()
        view = GtkView(canvas=c2)
        win.add(view)
        view.show()
        win.show()

        view.update()

    def test_pickle_demo(self):
        import demo

        canvas = demo.create_canvas()

        pickled = pickle.dumps(canvas)

        c2 = pickle.loads(pickled)

        win = Gtk.Window()
        view = GtkView(canvas=c2)
        win.add(view)

        view.show()
        win.show()

        view.update()


if __name__ == "__main__":
    unittest.main()

# vim: sw=4:et:ai



"""
Backport of the Python 3.0 weakref.WeakSet() class.

Note that, since it's shamelessly copied from the Python 3.0
distribution, this file is licensed under the Python Software
Foundation License, version 2.
"""

from builtins import object
from _weakref import ref

__all__ = ["WeakSet"]


class WeakSet(object):
    def __init__(self, data=None):
        self.data = set()

        def _remove(item, selfref=ref(self)):
            self = selfref()
            if self is not None:
                self.data.discard(item)

        self._remove = _remove
        if data is not None:
            self.update(data)

    def __iter__(self):
        for itemref in self.data:
            item = itemref()
            if item is not None:
                yield item

    def __len__(self):
        return sum(x() is not None for x in self.data)

    def __contains__(self, item):
        """
        >>> class C(object): pass
        >>> a = C()
        >>> b = C()
        >>> ws = WeakSet((a, b))
        >>> a in ws
        True
        >>> a = C()
        >>> a in ws
        False
        """
        return ref(item) in self.data

    def __reduce__(self):
        return (self.__class__, (list(self),), getattr(self, "__dict__", None))

    def add(self, item):
        self.data.add(ref(item, self._remove))

    def clear(self):
        """
        >>> class C(object): pass
        >>> s = C(), C()
        >>> ws = WeakSet(s)
        >>> list(ws)            # doctest: +ELLIPSIS
        [<gaphas.weakset.C object at 0x...>, <gaphas.weakset.C object at 0x...>]
        >>> ws.clear()
        >>> list(ws)
        []
        """
        self.data.clear()

    def copy(self):
        return self.__class__(self)

    def pop(self):
        """
        >>> class C(object): pass
        >>> a, b = C(), C()
        >>> ws = WeakSet((a, b))
        >>> len(ws)
        2
        >>> ws.pop()  # doctest: +ELLIPSIS
        <gaphas.weakset.C object at 0x...>
        >>> len(ws)
        1
        """

        while True:
            try:
                itemref = self.data.pop()
            except KeyError:
                raise KeyError("pop from empty WeakSet")
            item = itemref()
            if item is not None:
                return item

    def remove(self, item):
        self.data.remove(ref(item))

    def discard(self, item):
        self.data.discard(ref(item))

    def update(self, other):
        if isinstance(other, self.__class__):
            self.data.update(other.data)
        else:
            for element in other:
                self.add(element)

    def __ior__(self, other):
        self.update(other)
        return self

    # Helper functions for simple delegating methods.
    def _apply(self, other, method):
        if not isinstance(other, self.__class__):
            other = self.__class__(other)
        newdata = method(other.data)
        newset = self.__class__()
        newset.data = newdata
        return newset

    def difference(self, other):
        return self._apply(other, self.data.difference)

    __sub__ = difference

    def difference_update(self, other):
        if self is other:
            self.data.clear()
        else:
            self.data.difference_update(ref(item) for item in other)

    def __isub__(self, other):
        if self is other:
            self.data.clear()
        else:
            self.data.difference_update(ref(item) for item in other)
        return self

    def intersection(self, other):
        return self._apply(other, self.data.intersection)

    __and__ = intersection

    def intersection_update(self, other):
        self.data.intersection_update(ref(item) for item in other)

    def __iand__(self, other):
        self.data.intersection_update(ref(item) for item in other)
        return self

    def issubset(self, other):
        return self.data.issubset(ref(item) for item in other)

    __lt__ = issubset

    def __le__(self, other):
        return self.data <= set(ref(item) for item in other)

    def issuperset(self, other):
        return self.data.issuperset(ref(item) for item in other)

    __gt__ = issuperset

    def __ge__(self, other):
        return self.data >= set(ref(item) for item in other)

    def __eq__(self, other):
        """
        >>> class C(object): pass
        >>> a, b = C(), C()
        >>> ws1 = WeakSet((a, b))
        >>> ws2 = WeakSet((a, b))
        >>> ws1 == ws2
        True
        >>> ws1 == WeakSet((a, ))
        False
        """
        return self.data == set(ref(item) for item in other)

    def symmetric_difference(self, other):
        return self._apply(other, self.data.symmetric_difference)

    __xor__ = symmetric_difference

    def symmetric_difference_update(self, other):
        if self is other:
            self.data.clear()
        else:
            self.data.symmetric_difference_update(ref(item) for item in other)

    def __ixor__(self, other):
        if self is other:
            self.data.clear()
        else:
            self.data.symmetric_difference_update(ref(item) for item in other)
        return self

    def union(self, other):
        return self._apply(other, self.data.union)

    __or__ = union

    def isdisjoint(self, other):
        return len(self.intersection(other)) == 0


# vim:sw=4:et:ai



from builtins import range
from os import getenv

import unittest

from gaphas.canvas import Canvas
from gaphas.examples import Box

from gaphas.item import NW, NE, SE, SW


class ElementTestCase(unittest.TestCase):
    def test_creation_with_size(self):
        """
        Test if initial size holds when added to a canvas.
        """
        canvas = Canvas()
        box = Box(150, 153)

        assert box.width == 150, box.width
        assert box.height == 153, box.height
        assert box.handles()[SE].pos.x == 150, box.handles()[SE].pos.x
        assert box.handles()[SE].pos.y == 153, box.handles()[SE].pos.y

        canvas.add(box)

        assert box.width == 150, box.width
        assert box.height == 153, box.height
        assert box.handles()[SE].pos.x == 150, box.handles()[SE].pos.x
        assert box.handles()[SE].pos.y == 153, box.handles()[SE].pos.y

    def test_resize_se(self):
        """
        Test resizing of element by dragging it SE handle.
        """
        canvas = Canvas()
        box = Box()
        handles = box.handles()

        canvas.add(box)

        h_nw, h_ne, h_se, h_sw = handles
        assert h_nw is handles[NW]
        assert h_ne is handles[NE]
        assert h_sw is handles[SW]
        assert h_se is handles[SE]

        # to see how many solver was called:
        # GAPHAS_TEST_COUNT=3 nosetests -s --with-prof
        # --profile-restrict=gaphas gaphas/tests/test_element.py | grep -e
        # '\<solve\>' -e dirty

        count = getenv("GAPHAS_TEST_COUNT")
        if count:
            count = int(count)
        else:
            count = 1

        for i in range(count):
            h_se.pos.x += 100  # h.se.{x,y} = 10, now
            h_se.pos.y += 100
            box.request_update()
            canvas.update()

        self.assertEqual(
            110 * count, h_se.pos.x
        )  # h_se changed above, should remain the same
        self.assertEqual(110 * count, float(h_se.pos.y))

        self.assertEqual(110 * count, float(h_ne.pos.x))
        self.assertEqual(110 * count, float(h_sw.pos.y))

    def test_minimal_se(self):
        """
        Test resizing of element by dragging it SE handle.
        """
        canvas = Canvas()
        box = Box()
        handles = box.handles()

        canvas.add(box)

        h_nw, h_ne, h_se, h_sw = handles
        assert h_nw is handles[NW]
        assert h_ne is handles[NE]
        assert h_sw is handles[SW]
        assert h_se is handles[SE]

        h_se.pos.x -= 20  # h.se.{x,y} == -10
        h_se.pos.y -= 20
        assert h_se.pos.x == h_se.pos.y == -10

        box.request_update()
        canvas.update()

        self.assertEqual(10, h_se.pos.x)  # h_se changed above, should be 10
        self.assertEqual(10, h_se.pos.y)

        self.assertEqual(10, h_ne.pos.x)
        self.assertEqual(10, h_sw.pos.y)


# vim:sw=4:et:ai



"""
Generic gaphas item tests.
"""
from __future__ import print_function

import unittest

from gaphas.item import Item, Line
from gaphas.segment import *
from gaphas.canvas import Canvas
from gaphas.view import View
from gaphas import state
from gaphas.tests.test_tool import simple_canvas


class SegmentTestCase(unittest.TestCase):
    """
    Test aspects for items.
    """

    def setUp(self):
        self.canvas = Canvas()
        self.line = Line()
        self.view = View(self.canvas)

    def test_segment_fails_for_item(self):
        """
        Test if Segment aspect can be applied to Item
        """
        item = Item()
        try:
            s = Segment(item, self.view)
        except TypeError as e:
            pass
        else:
            assert False, "Should not be reached"

    def test_segment(self):
        """
        """
        view = self.view
        line = Line()
        self.canvas.add(line)
        segment = Segment(line, self.view)
        self.assertEqual(2, len(line.handles()))
        segment.split((5, 5))
        self.assertEqual(3, len(line.handles()))


undo_list = []
redo_list = []


def undo_handler(event):
    undo_list.append(event)


def undo():
    apply_me = list(undo_list)
    del undo_list[:]
    apply_me.reverse()
    for e in apply_me:
        state.saveapply(*e)
    redo_list[:] = undo_list[:]
    del undo_list[:]


class TestCaseBase(unittest.TestCase):
    """
    Abstract test case class with undo support.
    """

    def setUp(self):
        state.observers.add(state.revert_handler)
        state.subscribers.add(undo_handler)
        simple_canvas(self)

    def tearDown(self):
        state.observers.remove(state.revert_handler)
        state.subscribers.remove(undo_handler)


class LineSplitTestCase(TestCaseBase):
    """
    Tests for line splitting.
    """

    def test_split_single(self):
        """Test single line splitting
        """
        # we start with two handles and one port, after split 3 handles are
        # expected and 2 ports
        assert len(self.line.handles()) == 2
        assert len(self.line.ports()) == 1

        old_port = self.line.ports()[0]
        h1, h2 = self.line.handles()
        self.assertEqual(h1.pos, old_port.start)
        self.assertEqual(h2.pos, old_port.end)

        segment = Segment(self.line, self.view)

        handles, ports = segment.split_segment(0)
        handle = handles[0]
        self.assertEqual(1, len(handles))
        self.assertEqual((50, 50), handle.pos.pos)
        self.assertEqual(3, len(self.line.handles()))
        self.assertEqual(2, len(self.line.ports()))

        # new handle is between old handles
        self.assertEqual(handle, self.line.handles()[1])
        # and old port is deleted
        self.assertTrue(old_port not in self.line.ports())

        # check ports order
        p1, p2 = self.line.ports()
        h1, h2, h3 = self.line.handles()
        self.assertEqual(h1.pos, p1.start)
        self.assertEqual(h2.pos, p1.end)
        self.assertEqual(h2.pos, p2.start)
        self.assertEqual(h3.pos, p2.end)

    def test_split_multiple(self):
        """Test multiple line splitting
        """
        self.line.handles()[1].pos = (20, 16)
        handles = self.line.handles()
        old_ports = self.line.ports()[:]

        # start with two handles, split into 4 segments - 3 new handles to
        # be expected
        assert len(handles) == 2
        assert len(old_ports) == 1

        segment = Segment(self.line, self.view)

        handles, ports = segment.split_segment(0, count=4)
        self.assertEqual(3, len(handles))
        h1, h2, h3 = handles
        self.assertEqual((5, 4), h1.pos.pos)
        self.assertEqual((10, 8), h2.pos.pos)
        self.assertEqual((15, 12), h3.pos.pos)

        # new handles between old handles
        self.assertEqual(5, len(self.line.handles()))
        self.assertEqual(h1, self.line.handles()[1])
        self.assertEqual(h2, self.line.handles()[2])
        self.assertEqual(h3, self.line.handles()[3])

        self.assertEqual(4, len(self.line.ports()))

        # and old port is deleted
        self.assertTrue(old_ports[0] not in self.line.ports())

        # check ports order
        p1, p2, p3, p4 = self.line.ports()
        h1, h2, h3, h4, h5 = self.line.handles()
        self.assertEqual(h1.pos, p1.start)
        self.assertEqual(h2.pos, p1.end)
        self.assertEqual(h2.pos, p2.start)
        self.assertEqual(h3.pos, p2.end)
        self.assertEqual(h3.pos, p3.start)
        self.assertEqual(h4.pos, p3.end)
        self.assertEqual(h4.pos, p4.start)
        self.assertEqual(h5.pos, p4.end)

    def test_ports_after_split(self):
        """Test ports removal after split
        """
        self.line.handles()[1].pos = (20, 16)

        segment = Segment(self.line, self.view)

        segment.split_segment(0)
        handles = self.line.handles()
        old_ports = self.line.ports()[:]

        # start with 3 handles and two ports
        assert len(handles) == 3
        assert len(old_ports) == 2

        # do split of first segment again
        # first port should be deleted, but 2nd one should remain untouched
        segment.split_segment(0)
        self.assertFalse(old_ports[0] in self.line.ports())
        self.assertEqual(old_ports[1], self.line.ports()[2])

    def test_constraints_after_split(self):
        """Test if constraints are recreated after line split
        """

        # connect line2 to self.line
        line2 = Line()
        self.canvas.add(line2)
        head = line2.handles()[0]
        self.tool.connect(line2, head, (25, 25))
        cinfo = self.canvas.get_connection(head)
        self.assertEqual(self.line, cinfo.connected)

        Segment(self.line, self.view).split_segment(0)
        assert len(self.line.handles()) == 3
        h1, h2, h3 = self.line.handles()

        cinfo = self.canvas.get_connection(head)
        # connection shall be reconstrained between 1st and 2nd handle
        self.assertEqual(h1.pos, cinfo.constraint._line[0]._point)
        self.assertEqual(h2.pos, cinfo.constraint._line[1]._point)

    def test_split_undo(self):
        """Test line splitting undo
        """
        self.line.handles()[1].pos = (20, 0)

        # we start with two handles and one port, after split 3 handles and
        # 2 ports are expected
        assert len(self.line.handles()) == 2
        assert len(self.line.ports()) == 1

        segment = Segment(self.line, self.view)
        segment.split_segment(0)
        assert len(self.line.handles()) == 3
        assert len(self.line.ports()) == 2

        # after undo, 2 handles and 1 port are expected again
        undo()
        self.assertEqual(2, len(self.line.handles()))
        self.assertEqual(1, len(self.line.ports()))

    def test_orthogonal_line_split(self):
        """Test orthogonal line splitting
        """
        # start with no orthogonal constraints
        assert len(self.line._orthogonal_constraints) == 0

        segment = Segment(self.line, None)
        segment.split_segment(0)

        self.line.orthogonal = True

        # check orthogonal constraints
        self.assertEqual(2, len(self.line._orthogonal_constraints))
        self.assertEqual(3, len(self.line.handles()))

        Segment(self.line, self.view).split_segment(0)

        # 3 handles and 2 ports are expected
        # 2 constraints keep the self.line orthogonal
        self.assertEqual(3, len(self.line._orthogonal_constraints))
        self.assertEqual(4, len(self.line.handles()))
        self.assertEqual(3, len(self.line.ports()))

    def test_params_errors(self):
        """Test parameter error exceptions
        """
        line = Line()
        segment = Segment(line, self.view)

        # there is only 1 segment
        self.assertRaises(ValueError, segment.split_segment, -1)

        line = Line()
        segment = Segment(line, self.view)
        self.assertRaises(ValueError, segment.split_segment, 1)

        line = Line()
        # can't split into one or less segment :)
        segment = Segment(line, self.view)
        self.assertRaises(ValueError, segment.split_segment, 0, 1)


class LineMergeTestCase(TestCaseBase):
    """
    Tests for line merging.
    """

    def test_merge_first_single(self):
        """Test single line merging starting from 1st segment
        """
        self.line.handles()[1].pos = (20, 0)
        segment = Segment(self.line, self.view)
        segment.split_segment(0)

        # we start with 3 handles and 2 ports, after merging 2 handles and
        # 1 port are expected
        assert len(self.line.handles()) == 3
        assert len(self.line.ports()) == 2
        old_ports = self.line.ports()[:]

        segment = Segment(self.line, self.view)
        handles, ports = segment.merge_segment(0)
        # deleted handles and ports
        self.assertEqual(1, len(handles))
        self.assertEqual(2, len(ports))
        # handles and ports left after segment merging
        self.assertEqual(2, len(self.line.handles()))
        self.assertEqual(1, len(self.line.ports()))

        self.assertTrue(handles[0] not in self.line.handles())
        self.assertTrue(ports[0] not in self.line.ports())
        self.assertTrue(ports[1] not in self.line.ports())

        # old ports are completely removed as they are replaced by new one
        # port
        self.assertEqual(old_ports, ports)

        # finally, created port shall span between first and last handle
        port = self.line.ports()[0]
        self.assertEqual((0, 0), port.start.pos)
        self.assertEqual((20, 0), port.end.pos)

    def test_constraints_after_merge(self):
        """Test if constraints are recreated after line merge
        """

        # connect line2 to self.line
        line2 = Line()
        self.canvas.add(line2)
        head = line2.handles()[0]

        # conn = Connector(line2, head)
        # sink = conn.glue((25, 25))
        # assert sink is not None

        # conn.connect(sink)

        self.tool.connect(line2, head, (25, 25))
        cinfo = self.canvas.get_connection(head)
        self.assertEqual(self.line, cinfo.connected)

        segment = Segment(self.line, self.view)
        segment.split_segment(0)
        assert len(self.line.handles()) == 3
        c1 = cinfo.constraint

        segment.merge_segment(0)
        assert len(self.line.handles()) == 2

        h1, h2 = self.line.handles()
        # connection shall be reconstrained between 1st and 2nd handle
        cinfo = self.canvas.get_connection(head)
        self.assertEqual(cinfo.constraint._line[0]._point, h1.pos)
        self.assertEqual(cinfo.constraint._line[1]._point, h2.pos)
        self.assertFalse(c1 == cinfo.constraint)

    def test_merge_multiple(self):
        """Test multiple line merge
        """
        self.line.handles()[1].pos = (20, 16)
        segment = Segment(self.line, self.view)
        segment.split_segment(0, count=3)

        # start with 4 handles and 3 ports, merge 3 segments
        assert len(self.line.handles()) == 4
        assert len(self.line.ports()) == 3

        handles, ports = segment.merge_segment(0, count=3)
        self.assertEqual(2, len(handles))
        self.assertEqual(3, len(ports))
        self.assertEqual(2, len(self.line.handles()))
        self.assertEqual(1, len(self.line.ports()))

        self.assertTrue(not set(handles).intersection(set(self.line.handles())))
        self.assertTrue(not set(ports).intersection(set(self.line.ports())))

        # finally, created port shall span between first and last handle
        port = self.line.ports()[0]
        self.assertEqual((0, 0), port.start.pos)
        self.assertEqual((20, 16), port.end.pos)

    def test_merge_undo(self):
        """Test line merging undo
        """
        self.line.handles()[1].pos = (20, 0)

        segment = Segment(self.line, self.view)

        # split for merging
        segment.split_segment(0)
        assert len(self.line.handles()) == 3
        assert len(self.line.ports()) == 2

        # clear undo stack before merging
        del undo_list[:]

        # merge with empty undo stack
        segment.merge_segment(0)
        assert len(self.line.handles()) == 2
        assert len(self.line.ports()) == 1

        # after merge undo, 3 handles and 2 ports are expected again
        undo()
        self.assertEqual(3, len(self.line.handles()))
        self.assertEqual(2, len(self.line.ports()))

    def test_orthogonal_line_merge(self):
        """Test orthogonal line merging
        """
        self.assertEqual(4, len(self.canvas.solver._constraints))

        self.line.handles()[-1].pos = 100, 100

        segment = Segment(self.line, self.view)
        # prepare the self.line for merging
        segment.split_segment(0)
        segment.split_segment(0)
        self.line.orthogonal = True

        self.assertEqual(4 + 3, len(self.canvas.solver._constraints))
        self.assertEqual(4, len(self.line.handles()))
        self.assertEqual(3, len(self.line.ports()))

        # test the merging
        segment.merge_segment(0)

        self.assertEqual(4 + 2, len(self.canvas.solver._constraints))
        self.assertEqual(3, len(self.line.handles()))
        self.assertEqual(2, len(self.line.ports()))

    def test_params_errors(self):
        """Test parameter error exceptions
        """
        line = Line()
        self.canvas.add(line)
        segment = Segment(line, self.view)
        segment.split_segment(0)
        # no segment -1
        self.assertRaises(ValueError, segment.merge_segment, -1)

        line = Line()
        self.canvas.add(line)
        segment = Segment(line, self.view)
        segment.split_segment(0)
        # no segment no 2
        self.assertRaises(ValueError, segment.merge_segment, 2)

        line = Line()
        self.canvas.add(line)
        segment = Segment(line, self.view)
        segment.split_segment(0)
        # can't merge one or less segments :)
        self.assertRaises(ValueError, segment.merge_segment, 0, 1)

        line = Line()
        self.canvas.add(line)
        self.assertEqual(2, len(line.handles()))
        segment = Segment(line, self.view)
        # can't merge line with one segment
        self.assertRaises(ValueError, segment.merge_segment, 0)

        line = Line()
        self.canvas.add(line)
        segment = Segment(line, self.view)
        segment.split_segment(0)
        # 2 segments: no 0 and 1. cannot merge as there are no segments
        # after segment no 1
        self.assertRaises(ValueError, segment.merge_segment, 1)

        line = Line()
        self.canvas.add(line)
        segment = Segment(line, self.view)
        segment.split_segment(0)
        # 2 segments: no 0 and 1. cannot merge 3 segments as there are no 3
        # segments
        self.assertRaises(ValueError, segment.merge_segment, 0, 3)


class SegmentHandlesTest(unittest.TestCase):
    def setUp(self):
        self.canvas = Canvas()
        self.line = Line()
        self.canvas.add(self.line)
        self.view = View(self.canvas)

    def testHandleFinder(self):
        finder = HandleFinder(self.line, self.view)
        assert type(finder) is SegmentHandleFinder, type(finder)


# vim:sw=4:et:ai



"""
Generic gaphas item tests.
"""

import unittest

from gaphas.item import Item
from gaphas.constraint import LineConstraint, EqualsConstraint, LessThanConstraint
from gaphas.solver import Variable


class ItemConstraintTestCase(unittest.TestCase):
    """
    Item constraint creation tests. The test check functionality of
    `Item.constraint` method, not constraints themselves.
    """

    def test_line_constraint(self):
        """
        Test line creation constraint.
        """
        item = Item()
        pos = Variable(1), Variable(2)
        line = (Variable(3), Variable(4)), (Variable(5), Variable(6))
        item.constraint(line=(pos, line))
        self.assertEqual(1, len(item._constraints))

        c = item._constraints[0]
        self.assertTrue(isinstance(c, LineConstraint))
        self.assertEqual((1, 2), c._point)
        self.assertEqual(((3, 4), (5, 6)), c._line)

    def test_horizontal_constraint(self):
        """
        Test horizontal constraint creation.
        """
        item = Item()
        p1 = Variable(1), Variable(2)
        p2 = Variable(3), Variable(4)
        item.constraint(horizontal=(p1, p2))
        self.assertEqual(1, len(item._constraints))

        c = item._constraints[0]
        self.assertTrue(isinstance(c, EqualsConstraint))
        # expect constraint on y-axis
        self.assertEqual(2, c.a)
        self.assertEqual(4, c.b)

    def test_vertical_constraint(self):
        """
        Test vertical constraint creation.
        """
        item = Item()
        p1 = Variable(1), Variable(2)
        p2 = Variable(3), Variable(4)
        item.constraint(vertical=(p1, p2))
        self.assertEqual(1, len(item._constraints))

        c = item._constraints[0]
        self.assertTrue(isinstance(c, EqualsConstraint))
        # expect constraint on x-axis
        self.assertEqual(1, c.a)
        self.assertEqual(3, c.b)

    def test_left_of_constraint(self):
        """
        Test "less than" constraint (horizontal) creation.
        """
        item = Item()
        p1 = Variable(1), Variable(2)
        p2 = Variable(3), Variable(4)
        item.constraint(left_of=(p1, p2))
        self.assertEqual(1, len(item._constraints))

        c = item._constraints[0]
        self.assertTrue(isinstance(c, LessThanConstraint))
        self.assertEqual(1, c.smaller)
        self.assertEqual(3, c.bigger)

    def test_above_constraint(self):
        """
        Test "less than" constraint (vertical) creation.
        """
        item = Item()
        p1 = Variable(1), Variable(2)
        p2 = Variable(3), Variable(4)
        item.constraint(above=(p1, p2))
        self.assertEqual(1, len(item._constraints))

        c = item._constraints[0]
        self.assertTrue(isinstance(c, LessThanConstraint))
        self.assertEqual(2, c.smaller)
        self.assertEqual(4, c.bigger)



from __future__ import print_function

from builtins import object
import unittest
from gaphas.freehand import FreeHandCairoContext
import cairo


class PseudoFile(object):
    def __init__(self):
        self.data = ""

    def write(self, data):
        self.data = self.data + data


class FreeHandCairoContextTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_drawing_lines(self):
        f = PseudoFile()

        surface = cairo.SVGSurface("freehand-drawing-lines.svg", 100, 100)
        cr = FreeHandCairoContext(cairo.Context(surface))
        cr.set_line_width(2)
        cr.move_to(20, 20)
        cr.line_to(20, 80)
        cr.line_to(80, 80)
        cr.line_to(80, 20)
        cr.stroke()
        cr.show_page()

    def test_drawing_rectangle(self):
        surface = cairo.SVGSurface("freehand-drawing-rectangle.svg", 100, 100)
        cr = FreeHandCairoContext(cairo.Context(surface))
        cr.set_line_width(2)
        cr.rectangle(20, 20, 60, 60)
        cr.stroke()
        cr.show_page()


DRAWING_LINES_OUTPUT = """<?xml version="1.0" encoding="UTF-8"?>
<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="100pt" height="100pt" viewBox="0 0 100 100" version="1.1">
<g id="surface0">
<path style="fill:none;stroke-width:2;stroke-linecap:butt;stroke-linejoin:miter;stroke:rgb(0%,0%,0%);stroke-opacity:1;stroke-miterlimit:10;" d="M 20 20 C 23.324219 50.054688 17.195312 33.457031 20.722656 80.585938 C 38.78125 83.566406 20.984375 77.625 80.652344 80.652344 C 83.65625 70.992188 77.578125 60.988281 80.507812 20.019531 "/>
</g>
</svg>"""

# vim:sw=4:et:ai



"""
Unit tests for Gaphas' solver.
"""
from __future__ import print_function
from __future__ import division

import unittest
from timeit import Timer

from gaphas.solver import Solver, Variable
from gaphas.constraint import EquationConstraint, EqualsConstraint, LessThanConstraint


SETUP = """
from gaphas.solver import Solver, Variable
from gaphas.constraint import EqualsConstraint, LessThanConstraint
solver = Solver()
v1, v2, v3 = Variable(1.0), Variable(2.0), Variable(3.0)
c_eq = EqualsConstraint(v1, v2)
solver.add_constraint(c_eq)
"""

# Timeit constants
REPEAT = 30
NUMBER = 1000


class WeakestVariableTestCase(unittest.TestCase):
    """
    Test weakest variable calculation.
    """

    def test_weakest_list(self):
        """Test weakest list"""
        solver = Solver()
        a = Variable(1, 30)
        b = Variable(2, 10)
        c = Variable(3, 10)

        c_eq = EquationConstraint(lambda a, b, c: a + b + c, a=a, b=b, c=c)
        solver.add_constraint(c_eq)

        # because of kwargs above we cannot test by the order of arguments
        self.assertTrue(b in c_eq._weakest)
        self.assertTrue(c in c_eq._weakest)

    def test_weakest_list_order(self):
        """Test weakest list order"""
        solver = Solver()
        a = Variable(1, 30)
        b = Variable(2, 10)
        c = Variable(3, 10)

        c_eq = EquationConstraint(lambda a, b, c: a + b + c, a=a, b=b, c=c)
        solver.add_constraint(c_eq)

        weakest = [el for el in c_eq._weakest]
        a.value = 4

        self.assertEqual(
            c_eq._weakest, weakest
        )  # does not change if non-weak variable changed

        b.value = 5
        self.assertEqual(c_eq.weakest(), c)

        b.value = 6
        self.assertEqual(c_eq.weakest(), c)

        # b changed above, now change a - all weakest variables changed
        # return the oldest changed variable then
        c.value = 6
        self.assertEqual(c_eq.weakest(), b)

        b.value = 6
        self.assertEqual(c_eq.weakest(), c)

    def test_strength_change(self):
        """Test strength change"""
        solver = Solver()
        a = Variable(1, 30)
        b = Variable(2, 10)
        c = Variable(3, 10)

        c_eq = EquationConstraint(lambda a, b, c: a + b + c, a=a, b=b, c=c)
        solver.add_constraint(c_eq)

        b.strength = 9
        self.assertEqual(c_eq._weakest, [b])


class SizeTestCase(unittest.TestCase):
    """
    Test size related constraints, i.e. minimal size.
    """

    def test_min_size(self):
        """Test minimal size constraint"""
        solver = Solver()
        v1 = Variable(0)
        v2 = Variable(10)
        v3 = Variable(10)
        c1 = EqualsConstraint(a=v2, b=v3)
        c2 = LessThanConstraint(smaller=v1, bigger=v3, delta=10)
        solver.add_constraint(c1)
        solver.add_constraint(c2)

        # check everyting is ok on start
        solver.solve()
        self.assertEqual(0, v1)
        self.assertEqual(10, v2)
        self.assertEqual(10, v3)

        # change v1 to 2, after solve it should be 0 again due to LT
        # constraint
        v1.value = 2
        solver.solve()

        self.assertEqual(0, v1)
        self.assertEqual(10, v2)
        self.assertEqual(10, v3)

        # change v3 to 20, after solve v2 will follow thanks to EQ
        # constraint
        v3.value = 20
        solver.solve()

        self.assertEqual(0, v1)
        self.assertEqual(20, v2)
        self.assertEqual(20, v3)

        # change v3 to 0, after solve it shoul be 10 due to LT.delta = 10,
        # v2 should also be 10 due to EQ constraint
        v3.value = 0
        solver.solve()

        self.assertEqual(0, v1)
        self.assertEqual(10, v2)
        self.assertEqual(10, v3)


class SolverSpeedTestCase(unittest.TestCase):
    """
    Solver speed tests.
    """

    def _test_speed_run_weakest(self):
        """
        Speed test for weakest variable.
        """

        results = Timer(
            setup=SETUP,
            stmt="""
v1.value = 5.0
c_eq.weakest()""",
        ).repeat(repeat=REPEAT, number=NUMBER)

        # Print the average of the best 10 runs:
        results.sort()
        print("[Avg: %gms]" % ((sum(results[:10]) / 10)) * 1000)


if __name__ == "__main__":
    unittest.main()

# vim:sw=4:et:ai



"""
Gaphas
======

Gaphor's Canvas.

This module contains the application independant parts of Gaphor's
Canvas.  It can and may be used by others under the terms of the GNU
LGPL licence.

Notes
=====

In py-cairo 1.8.0 (or 1.8.1, or 1.8.2) the multiplication order has
been reverted. This causes bugs in Gaphas.

Also a new method ``multiply()`` has been introduced. This method is
used in Gaphas instead of the multiplier (``*``). In both the
``Canvas`` and ``View`` class a workaround is provided in case an
older version of py-cairo is used.
"""
from __future__ import absolute_import

__version__ = "$Revision$"
# $HeadURL$


from .canvas import Canvas
from .connector import Handle
from .item import Item, Line, Element
from .view import View, GtkView

# vi:sw=4:et:ai



import unittest

from gaphas.solver import Variable
from gaphas.constraint import PositionConstraint, LineAlignConstraint


class PositionTestCase(unittest.TestCase):
    def test_pos_constraint(self):
        """Test position constraint"""
        x1, y1 = Variable(10), Variable(11)
        x2, y2 = Variable(12), Variable(13)
        pc = PositionConstraint(origin=(x1, y1), point=(x2, y2))
        pc.solve_for()

        # origin shall remain the same
        self.assertEqual(10, x1)
        self.assertEqual(11, y1)

        # point shall be moved to origin
        self.assertEqual(10, x2)
        self.assertEqual(11, y2)

        # change just x of origin
        x1.value = 15
        pc.solve_for()
        self.assertEqual(15, x2)

        # change just y of origin
        y1.value = 14
        pc.solve_for()
        self.assertEqual(14, y2)


class LineAlignConstraintTestCase(unittest.TestCase):
    """
    Line align constraint test case.
    """

    def test_delta(self):
        """Test line align delta
        """
        line = (Variable(0), Variable(0)), (Variable(30), Variable(20))
        point = (Variable(15), Variable(10))
        lc = LineAlignConstraint(line=line, point=point, align=0.5, delta=5)
        lc.solve_for()
        self.assertAlmostEqual(19.16, point[0].value, 2)
        self.assertAlmostEqual(12.77, point[1].value, 2)

        line[1][0].value = 40
        line[1][1].value = 30
        lc.solve_for()
        self.assertAlmostEqual(24.00, point[0].value, 2)
        self.assertAlmostEqual(18.00, point[1].value, 2)

    def test_delta_below_zero(self):
        """Test line align with delta below zero
        """
        line = (Variable(0), Variable(0)), (Variable(30), Variable(20))
        point = (Variable(15), Variable(10))
        lc = LineAlignConstraint(line=line, point=point, align=0.5, delta=-5)
        lc.solve_for()
        self.assertAlmostEqual(10.84, point[0].value, 2)
        self.assertAlmostEqual(7.23, point[1].value, 2)

        line[1][0].value = 40
        line[1][1].value = 30
        lc.solve_for()
        self.assertAlmostEqual(16.0, point[0].value, 2)
        self.assertAlmostEqual(12.00, point[1].value, 2)


# vim: sw=4:et:ai



import unittest
from gaphas.tree import Tree


class TreeTestCase(unittest.TestCase):
    def test_add(self):
        """
        Test creating node trees.
        """
        tree = Tree()
        n1 = "n1"
        n2 = "n2"
        n3 = "n3"

        tree.add(n1)
        assert len(tree._nodes) == 1
        assert len(tree._children) == 2
        assert len(tree._children[None]) == 1
        assert len(tree._children[n1]) == 0

        tree.add(n2)
        tree.add(n3, parent=n1)
        assert len(tree._nodes) == 3
        assert len(tree._children) == 4
        assert len(tree._children[None]) == 2
        assert len(tree._children[n1]) == 1
        assert len(tree._children[n2]) == 0
        assert len(tree._children[n2]) == 0
        assert tree._nodes == [n1, n3, n2]

        n4 = "n4"
        tree.add(n4, parent=n3)
        assert tree._nodes == [n1, n3, n4, n2]

        n5 = "n5"
        tree.add(n5, parent=n3)
        assert tree._nodes == [n1, n3, n4, n5, n2]

        n6 = "n6"
        tree.add(n6, parent=n2)
        assert tree._nodes == [n1, n3, n4, n5, n2, n6]

        n7 = "n7"
        tree.add(n7, parent=n1)
        assert len(tree._children) == 8
        assert tree._nodes == [n1, n3, n4, n5, n7, n2, n6]
        assert tree.get_parent(n7) is n1
        assert tree.get_parent(n6) is n2
        assert tree.get_parent(n5) is n3
        assert tree.get_parent(n4) is n3
        assert tree.get_parent(n3) is n1
        assert tree.get_parent(n2) is None
        assert tree.get_parent(n1) is None

    def test_add_on_index(self):
        tree = Tree()
        n1 = "n1"
        n2 = "n2"
        n3 = "n3"
        n4 = "n4"
        n5 = "n5"

        tree.add(n1)
        tree.add(n2)
        tree.add(n3, index=1)
        assert tree.get_children(None) == [n1, n3, n2], tree.get_children(None)
        assert tree.nodes == [n1, n3, n2], tree.nodes

        tree.add(n4, parent=n3)
        tree.add(n5, parent=n3, index=0)
        assert tree.get_children(None) == [n1, n3, n2], tree.get_children(None)
        assert tree.nodes == [n1, n3, n5, n4, n2], tree.nodes
        assert tree.get_children(n3) == [n5, n4], tree.get_children(n3)

    def test_remove(self):
        """
        Test removal of nodes.
        """
        tree = Tree()
        n1 = "n1"
        n2 = "n2"
        n3 = "n3"
        n4 = "n4"
        n5 = "n5"

        tree.add(n1)
        tree.add(n2)
        tree.add(n3, parent=n1)
        tree.add(n4, parent=n3)
        tree.add(n5, parent=n4)

        assert tree._nodes == [n1, n3, n4, n5, n2]

        all_ch = list(tree.get_all_children(n1))
        assert all_ch == [n3, n4, n5], all_ch

        tree.remove(n4)
        assert tree._nodes == [n1, n3, n2]

        tree.remove(n1)
        assert len(tree._children) == 2
        assert tree._children[None] == [n2]
        assert tree._children[n2] == []
        assert tree._nodes == [n2]

    def test_siblings(self):
        tree = Tree()
        n1 = "n1"
        n2 = "n2"
        n3 = "n3"

        tree.add(n1)
        tree.add(n2)
        tree.add(n3)

        assert tree.get_next_sibling(n1) is n2
        assert tree.get_next_sibling(n2) is n3
        try:
            tree.get_next_sibling(n3)
        except IndexError:
            pass  # okay
        else:
            raise AssertionError(
                "Index should be out of range, not %s" % tree.get_next_sibling(n3)
            )

        assert tree.get_previous_sibling(n3) is n2
        assert tree.get_previous_sibling(n2) is n1
        try:
            tree.get_previous_sibling(n1)
        except IndexError:
            pass  # okay
        else:
            raise AssertionError(
                "Index should be out of range, not %s" % tree.get_previous_sibling(n1)
            )

    def test_reparent(self):
        tree = Tree()
        n1 = "n1"
        n2 = "n2"
        n3 = "n3"
        n4 = "n4"
        n5 = "n5"

        tree.add(n1)
        tree.add(n2)
        tree.add(n3)
        tree.add(n4, parent=n2)
        tree.add(n5, parent=n4)

        assert tree.nodes == [n1, n2, n4, n5, n3], tree.nodes

        tree.reparent(n4, parent=n1, index=0)
        assert tree.nodes == [n1, n4, n5, n2, n3], tree.nodes
        assert tree.get_children(n2) == [], tree.get_children(n2)
        assert tree.get_children(n1) == [n4], tree.get_children(n1)
        assert tree.get_children(n4) == [n5], tree.get_children(n4)

        tree.reparent(n4, parent=None, index=0)
        assert tree.nodes == [n4, n5, n1, n2, n3], tree.nodes


# vi:sw=4:et:ai



from builtins import range
import unittest
from gaphas.quadtree import Quadtree


class QuadtreeTestCase(unittest.TestCase):
    def test_lookups(self):
        qtree = Quadtree((0, 0, 100, 100))
        for i in range(100, 10):
            for j in range(100, 10):
                qtree.add("%dx%d" % (i, j), (i, j, 10, 10))

        for i in range(100, 10):
            for j in range(100, 10):
                assert qtree.find_intersect((i + 1, j + 1, 1, 1)) == [
                    "%dx%d" % (i, j)
                ], qtree.find_intersect((i + 1, j + 1, 1, 1))

    def test_with_rectangles(self):
        from gaphas.geometry import Rectangle

        qtree = Quadtree((0, 0, 100, 100))
        for i in range(0, 100, 10):
            for j in range(0, 100, 10):
                qtree.add("%dx%d" % (i, j), Rectangle(i, j, 10, 10))
        assert len(qtree._ids) == 100, len(qtree._ids)

        for i in range(100, 10):
            for j in range(100, 10):
                assert qtree.find_intersect((i + 1, j + 1, 1, 1)) == [
                    "%dx%d" % (i, j)
                ], qtree.find_intersect((i + 1, j + 1, 1, 1))

    def test_moving_items(self):
        qtree = Quadtree((0, 0, 100, 100), capacity=10)
        for i in range(0, 100, 10):
            for j in range(0, 100, 10):
                qtree.add("%dx%d" % (i, j), (i, j, 10, 10))
        assert len(qtree._ids) == 100, len(qtree._ids)
        assert qtree._bucket._buckets, qtree._bucket._buckets
        for i in range(4):
            assert qtree._bucket._buckets[i]._buckets
            for j in range(4):
                assert not qtree._bucket._buckets[i]._buckets[j]._buckets

        # Check contents:
        # First sub-level contains 9 items. second level contains 4 items
        # ==> 4 * (9 + (4 * 4)) = 100
        assert len(qtree._bucket.items) == 0, qtree._bucket.items
        for i in range(4):
            assert len(qtree._bucket._buckets[i].items) == 9
            for item, bounds in qtree._bucket._buckets[i].items.items():
                assert qtree._bucket.find_bucket(bounds) is qtree._bucket._buckets[i]
            for j in range(4):
                assert len(qtree._bucket._buckets[i]._buckets[j].items) == 4

        assert qtree.get_bounds("0x0")
        # Now move item '0x0' to the center of the first quadrant (20, 20)
        qtree.add("0x0", (20, 20, 10, 10))
        assert len(qtree._bucket.items) == 0
        assert len(qtree._bucket._buckets[0]._buckets[0].items) == 3, (
            qtree._bucket._buckets[0]._buckets[0].items
        )
        assert len(qtree._bucket._buckets[0].items) == 10, qtree._bucket._buckets[
            0
        ].items

        # Now move item '0x0' to the second quadrant (70, 20)
        qtree.add("0x0", (70, 20, 10, 10))
        assert len(qtree._bucket.items) == 0
        assert len(qtree._bucket._buckets[0]._buckets[0].items) == 3, (
            qtree._bucket._buckets[0]._buckets[0].items
        )
        assert len(qtree._bucket._buckets[0].items) == 9, qtree._bucket._buckets[
            0
        ].items
        assert len(qtree._bucket._buckets[1].items) == 10, qtree._bucket._buckets[
            1
        ].items

    def test_get_data(self):
        """
        Extra data may be added to a node:
        """
        qtree = Quadtree((0, 0, 100, 100))
        for i in range(0, 100, 10):
            for j in range(0, 100, 10):
                qtree.add("%dx%d" % (i, j), (i, j, 10, 10), i + j)

        for i in range(0, 100, 10):
            for j in range(0, 100, 10):
                assert i + j == qtree.get_data("%dx%d" % (i, j))

    def test_clipped_bounds(self):
        qtree = Quadtree((0, 0, 100, 100), capacity=10)
        qtree.add(1, (-100, -100, 120, 120))
        self.assertEqual((0, 0, 20, 20), qtree.get_clipped_bounds(1))


if __name__ == "__main__":
    unittest.main()

# vim:sw=4:et:ai



import unittest
from gaphas.connector import Position, Handle
from gaphas.solver import Variable


class PositionTestCase(unittest.TestCase):
    def test_position(self):
        pos = Position((0, 0))
        self.assertEqual(0, pos.x)
        self.assertEqual(0, pos.y)

    def test_position(self):
        pos = Position((1, 2))
        self.assertEqual(1, pos.x)
        self.assertEqual(2, pos.y)

    def test_set_xy(self):
        pos = Position((1, 2))
        x = Variable()
        y = Variable()
        assert x is not pos.x
        assert y is not pos.y

        pos.set_x(x)
        pos.set_y(y)
        assert x is pos.x
        assert y is pos.y


class HandleTestCase(unittest.TestCase):
    def test_handle_x_y(self):
        h = Handle()
        self.assertEqual(0.0, h.x)
        self.assertEqual(0.0, h.y)


# vim: sw=4:et:ai



import unittest

from gaphas import state
from gaphas.aspect import Connector, ConnectionSink
from gaphas.canvas import Canvas
from gaphas.examples import Box
from gaphas.item import Line

state.observers.clear()
state.subscribers.clear()

undo_list = []
redo_list = []


def undo_handler(event):
    undo_list.append(event)


def undo():
    apply_me = list(undo_list)
    del undo_list[:]
    apply_me.reverse()
    for e in apply_me:
        state.saveapply(*e)
    redo_list[:] = undo_list[:]
    del undo_list[:]


class UndoTestCase(unittest.TestCase):
    def setUp(self):
        state.observers.add(state.revert_handler)
        state.subscribers.add(undo_handler)

    def shutDown(self):
        state.observers.remove(state.revert_handler)
        state.subscribers.remove(undo_handler)

    def testUndoOnDeletedElement(self):
        b1 = Box()

        b2 = Box()
        line = Line()

        canvas = Canvas()
        canvas.add(b1)
        self.assertEqual(2, len(canvas.solver.constraints))

        canvas.add(b2)
        self.assertEqual(4, len(canvas.solver.constraints))

        canvas.add(line)

        sink = ConnectionSink(b1, b1.ports()[0])
        connector = Connector(line, line.handles()[0])
        connector.connect(sink)

        sink = ConnectionSink(b2, b2.ports()[0])
        connector = Connector(line, line.handles()[-1])
        connector.connect(sink)

        self.assertEqual(6, len(canvas.solver.constraints))
        self.assertEqual(2, len(list(canvas.get_connections(item=line))))

        del undo_list[:]

        # Here disconnect is not invoked!
        canvas.remove(b2)

        self.assertEqual(3, len(canvas.solver.constraints))
        self.assertEqual(1, len(list(canvas.get_connections(item=line))))

        cinfo = canvas.get_connection(line.handles()[0])
        self.assertEqual(b1, cinfo.connected)

        cinfo = canvas.get_connection(line.handles()[-1])
        self.assertEqual(None, cinfo)

        self.assertEqual(
            [], list(canvas.solver.constraints_with_variable(line.handles()[-1].pos.x))
        )
        self.assertEqual(
            [], list(canvas.solver.constraints_with_variable(line.handles()[-1].pos.y))
        )

        undo()

        self.assertEqual(6, len(canvas.solver.constraints))
        self.assertEqual(2, len(list(canvas.get_connections(item=line))))

        cinfo = canvas.get_connection(line.handles()[0])
        self.assertEqual(b1, cinfo.connected)

        cinfo = canvas.get_connection(line.handles()[-1])
        self.assertEqual(b2, cinfo.connected)


#        self.assertEqual(list(canvas.solver.constraints_with_variable(line.handles()[-1].pos.x)))
#        self.assertTrue(list(canvas.solver.constraints_with_variable(line.handles()[-1].pos.y)))

if __name__ == "__main__":
    unittest.main()
# vim:sw=4:et:ai



from __future__ import print_function

import unittest
from builtins import range

from gi.repository import Gtk

from gaphas.canvas import Canvas
from gaphas.guide import Guide, GuidedItemInMotion
from gaphas.item import Element, Line
from gaphas.view import GtkView


class GuideTestCase(unittest.TestCase):
    def setUp(self):
        self.canvas = Canvas()
        self.view = GtkView(self.canvas)
        self.window = Gtk.Window()
        self.window.add(self.view)
        self.window.show_all()

    def tearDown(self):
        self.window.destroy()

    def test_find_closest(self):
        """
        test find closest method.
        """
        set1 = [0, 10, 20]
        set2 = [2, 15, 30]

        guider = GuidedItemInMotion(Element(), self.view)
        d, closest = guider.find_closest(set1, set2)
        self.assertEqual(2.0, d)
        self.assertEqual([2.0], closest)

    def test_element_guide(self):
        e1 = Element()
        self.assertEqual(10, e1.width)
        self.assertEqual(10, e1.height)
        guides = Guide(e1).horizontal()
        self.assertEqual(0.0, guides[0])
        self.assertEqual(5.0, guides[1])
        self.assertEqual(10.0, guides[2])
        guides = Guide(e1).vertical()
        self.assertEqual(0.0, guides[0])
        self.assertEqual(5.0, guides[1])
        self.assertEqual(10.0, guides[2])

    def test_line_guide(self):
        c = Canvas()
        line = Line()
        c.add(line)
        line.handles().append(line._create_handle((20, 20)))
        line.handles().append(line._create_handle((30, 30)))
        line.handles().append(line._create_handle((40, 40)))
        line.orthogonal = True
        c.update_now()

        guides = list(Guide(line).horizontal())
        self.assertEqual(2, len(guides))
        self.assertEqual(10.0, guides[0])
        self.assertEqual(40.0, guides[1])

        guides = list(Guide(line).vertical())
        self.assertEqual(2, len(guides))
        self.assertEqual(00.0, guides[0])
        self.assertEqual(20.0, guides[1])

    def test_line_guide_horizontal(self):
        c = Canvas()
        line = Line()
        c.add(line)
        line.handles().append(line._create_handle((20, 20)))
        line.handles().append(line._create_handle((30, 30)))
        line.handles().append(line._create_handle((40, 40)))
        line.horizontal = True
        line.orthogonal = True
        c.update_now()

        guides = list(Guide(line).horizontal())
        self.assertEqual(2, len(guides))
        self.assertEqual(0.0, guides[0])
        self.assertEqual(20.0, guides[1])

        guides = list(Guide(line).horizontal())
        self.assertEqual(2, len(guides))
        self.assertEqual(0.0, guides[0])
        self.assertEqual(20.0, guides[1])

    def test_guide_item_in_motion(self):
        e1 = Element()
        e2 = Element()
        e3 = Element()

        canvas = self.canvas
        canvas.add(e1)
        canvas.add(e2)
        canvas.add(e3)

        self.assertEqual(0, e1.matrix[4])
        self.assertEqual(0, e1.matrix[5])

        e2.matrix.translate(40, 40)
        e2.request_update()
        self.assertEqual(40, e2.matrix[4])
        self.assertEqual(40, e2.matrix[5])

        guider = GuidedItemInMotion(e3, self.view)

        guider.start_move((0, 0))
        self.assertEqual(0, e3.matrix[4])
        self.assertEqual(0, e3.matrix[5])

        # Moved back to guided lines:
        for d in range(0, 3):
            guider.move((d, d))
            self.assertEqual(0, e3.matrix[4])
            self.assertEqual(0, e3.matrix[5])

        for d in range(3, 5):
            guider.move((d, d))
            self.assertEqual(5, e3.matrix[4])
            self.assertEqual(5, e3.matrix[5])

        guider.move((20, 20))
        self.assertEqual(20, e3.matrix[4])
        self.assertEqual(20, e3.matrix[5])

    def test_guide_item_in_motion_2(self):
        e1 = Element()
        e2 = Element()
        e3 = Element()

        canvas = self.canvas
        canvas.add(e1)
        canvas.add(e2)
        canvas.add(e3)

        self.assertEqual(0, e1.matrix[4])
        self.assertEqual(0, e1.matrix[5])

        e2.matrix.translate(40, 40)
        e2.request_update()
        self.assertEqual(40, e2.matrix[4])
        self.assertEqual(40, e2.matrix[5])

        guider = GuidedItemInMotion(e3, self.view)

        guider.start_move((3, 3))
        self.assertEqual(0, e3.matrix[4])
        self.assertEqual(0, e3.matrix[5])

        # Moved back to guided lines:
        for y in range(4, 6):
            guider.move((3, y))
            self.assertEqual(0, e3.matrix[4])
            self.assertEqual(0, e3.matrix[5])

        for y in range(6, 9):
            guider.move((3, y))
            self.assertEqual(0, e3.matrix[4])
            self.assertEqual(5, e3.matrix[5])

        # Take into account initial cursor offset of (3, 3)
        guider.move((20, 23))
        self.assertEqual(17, e3.matrix[4])
        self.assertEqual(20, e3.matrix[5])


# vim:sw=4:et:ai



"""
Generic gaphas item tests.
"""

import unittest

from gaphas.aspect import *
from gaphas.canvas import Canvas
from gaphas.view import View


class AspectTestCase(unittest.TestCase):
    """
    Test aspects for items.
    """

    def setUp(self):
        self.canvas = Canvas()
        self.view = View(self.canvas)

    def test_selection_select(self):
        """
        Test the Selection role methods
        """
        view = self.view
        item = Item()
        self.canvas.add(item)
        selection = Selection(item, view)
        assert item not in view.selected_items
        selection.select()
        assert item in view.selected_items
        assert item is view.focused_item
        selection.unselect()
        assert item not in view.selected_items
        assert None is view.focused_item

    def test_selection_move(self):
        """
        Test the Selection role methods
        """
        view = self.view
        item = Item()
        self.canvas.add(item)
        inmotion = InMotion(item, view)
        self.assertEqual((1, 0, 0, 1, 0, 0), tuple(item.matrix))
        inmotion.start_move((0, 0))
        inmotion.move((12, 26))
        self.assertEqual((1, 0, 0, 1, 12, 26), tuple(item.matrix))


# vim:sw=4:et:ai



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['gaphas', 'gaphas.tests']

package_data = \
{'': ['*']}

install_requires = \
['PyGObject>=3.20.0,<4.0.0',
 'decorator>=4.3.0,<5.0.0',
 'future>=0.17.0,<0.18.0',
 'pycairo>=1.10.0,<2.0.0',
 'simplegeneric>=0.8.1,<0.9.0']

entry_points = \
{'console_scripts': ['poetry = poetry.console:main']}

setup_kwargs = {
    'name': 'gaphas',
    'version': '1.0.0rc1',
    'description': 'Gaphas is a GTK+ based diagramming widget',
    'long_description': 'Gaphas\n======\n\n|Code style: black|\n|Build status|\n|Coverage|\n\nThis module contains a new canvas implementation for Gaphor.\n\nThe basic idea is:\n\n- Items (canvas items) can be added to a Canvas.\n- The canvas maintains the tree structure (parent-child relationships between\n  items).\n- A constraint solver is used to maintain item constraints and inter-item\n  constraints.\n- The item (and user) should not be bothered with things like bounding-box\n  calculations.\n- Very modular: e.g. handle support could be swapped in and swapped out.\n- Rendering using Cairo.\n\n Gaphas is released under the terms of the GNU Library General Public License\n (LGPL). See the COPYING file for details.\n\n.. contents::\n\n\nHow it Works\n============\n\nThe Canvas class (from canvas.py) acts as a container for Item\'s (from item.py).\nThe item\'s parent/child relationships are maintained here (not in the Item!).\n\nAn Item can have a set of Handle\'s (from connector.py) which can be used to\nmanipulate the item (although this is not necessary). Each item has it\'s own\ncoordinate system (a (0, 0) point). Item.matrix is the transformation\nrelative to the parent item of the Item, as defined in the Canvas.\nHandles can connect to Ports. A Port is a location (line or point) where a\nhandle is allowed to connect on another item. The process of connecting\ndepends on the case at hand, but most often involves the creation of some\nsort of constraint between the Handle and the item it\'s connecting to (see\ndoc/ports.txt).\n\nThe Canvas also contains a constraint Solver (from solver.py) that can be used\nto solve mathematical dependencies between items (such as Handles that should\nbe aligned). The constraint solver is also a handy tool to keep constraint\nin the item true (e.g. make sure a box maintains it\'s rectangular shape).\n\nView (from view.py) is used to visualize a canvas. On a View, a Tool\n(from tool.py) can be applied, which will handle user input (button presses,\nkey presses, etc.). Painters (from painter.py) are used to do the actual\ndrawing. This way it should be easy do draw to other media than the screen,\nsuch as a printer or PDF document.\n\nUpdating item state\n-------------------\nIf an items needs updating, it sets out an update request on the Canvas\n(Canvas.request_update()). The canvas performs an update by calling:\n\n1. Item.pre_update(context) for each item marked for update\n2. Updating Canvas-to-Item matrices, for fast transformation of coordinates\n   from the canvas\' to the items\' coordinate system.\n   The c2i matrix is stored on the Item as Item._matrix_c2i.\n3. Solve constraints.\n4. Normalize items by setting the coordinates of the first handle to (0, 0).\n5. Updating Canvas-to-Item matrices for items that have been changed by\n   normalization, just to be on the save side.\n6. Item.post_update(context) for each item marked for update, including items\n   that have been marked during constraint solving.\n\nThe idea is to do as much updating as possible in the {pre|post}_update()\nmethods, since they are called when the application is not handling user input.\n\nThe context contains:\n\n:cairo: a CairoContext, this can be used to calculate the dimensions of text\n        for example\n\nNOTE: updating is done from the canvas, items should not update sub-items.\n\nAfter an update, the Item should be ready to be drawn.\n\nConstraint solving\n------------------\nA word about the constraint solver seems in place. It is one of the big\nfeatures of this library after all. The Solver is able to solve constraints.\nConstraints can be applied to items (Variables owned by the item actually).\nElement items, for example, uses constraints to maintain their recangular\nshape. Constraints can be created *between* items (for example a line that\nconnects to a box).\n\nConstraints that apply to one item are pretty straight forward, as all variables\nlive in the same coordinate system (of the item). The variables (in most cases\na Handle\'s x and y coordinate) can simply be put in a constraint.\n\nWhen two items are connected to each other and constraints are created, a\nproblem shows up: variables live in separate coordinate systems. To overcome\nthis problem a Projection (from solver.py) has been defined. With a Projection\ninstance, a variable can be "projected" on another coordinate system. In this\ncase, where two items are connecting to each other, the Canvas\' coordinate\nsystem is used.\n\n\nDrawing\n-------\nDrawing is done by the View. All items marked for redraw (e.i. the items\nthat had been updated) will be drawn in the order in which they reside in the\nCanvas (first root item, then it\'s children; second root item, etc.)\n\nThe view context passed to the Items draw() method has the following properties:\n\n:view:     the view we\'re drawing to\n:cairo:    the CairoContext to draw to\n:selected: True if the item is actually selected in the view\n:focused:  True if the item has the focus\n:hovered:  True if the mouse pointer if over the item. Only the top-most item\n           is marked as hovered.\n:dropzone: The item is marked as drop zone. This happens then an item is\n           dragged over the item and (if dropped) will become a child of\n           this item.\n:draw_all: True if everything drawable on the item should be drawn (e.g. when\n           calculating the bounding boxes).\n\nThe View automatically calculates the bounding box for the item, based on the\nitems drawn in the draw(context) function (this is only done when really\nnecessary, e.g. after an update of the item). The bounding box is in viewport\ncoordinates.\n\nThe actual drawing is done by Painters (painter.py). A series of Painters have\nbeen defined: one for handles, one for items, etc.\n\nTools\n-----\nBehaviour is added to the canvas(-view) by tools.\n\nTools can be chained together in order to provide more complex behaviour.\n\nTo make it easy a DefaultTool has been defined: a ToolChain instance with the\ntools added that are listed in the following sections.\n\nToolChain\n    The ToolChain does not do anything by itself. It delegates to a set of\n    tools and keeps track of which tool has grabbed the focus. This happens\n    most of the time when the uses presses a mouse button. The tool requests a\n    grab() and all upcoming events (e.g. motion or button release events) are\n    directly sent to the focused tool.\n\nHoverTool\n    A small and simple tool that does nothing more than making the item under\n    the mouse button the "hovered item". When such an item is drawn, its\n    context.hovered_item flag will be set to True.\n\nHandleTool\n    The HandleTool is used to deal with handles. Handles can be dragged around.\n    Clicking on a handle automatically makes the underlying item the focused\n    item.\n\nItemTool\n    The item tool takes care of selecting items and dragging items around.\n\nTextEditTool\n    This is a demo-tool, featuring a text-edit pop-up.\n\nRubberbandTool\n    The last tool in line is the rubber band tool. It\'s invoked when the mouse\n    button is pressed on a section of the view where no items or handles are\n    present. It allows the user to select items using a selection box\n    (rubber band).\n\n\nInteraction\n-----------\nInteraction with the canvas view (visual component) is handled by tools.\nAlthough the default tools do a fair amount of work, in most cases you\'ll\nsee that especially the way items connect with each other is not the way\nyou want it. That\'s okay. HandleTool provides some hooks (connect, disconnect\nand glue) to implement custom connection behaviour (in fact, the default\nimplementation doesn\'t do any connecting at all!).\n\nOne of the problems you\'ll face is what to do when an item is removed from the\ncanvas and there are other items (lines) connected to. This problem can be\novercome by providing a disconnect handler to the handle instance ones it is\nconnected. A callable object (e.g. function) can be assigned to the handle. It\nis called at the moment the item it\'s connected to is removed from the canvas.\n\n\nUndo\n====\n\nGaphas has a simple build-in system for registering changes in it\'s classes and\nnotifying the application. This code resides in state.py.\n\nThere is also a "reverter" framework in place. This "framework" is notified\nwhen objects change their state and will figure out the reverse operation that\nhas to be applied in order to undo the operation.\n\nSee state.txt and undo.txt for details and usage examples.\n\n\nGuidelines\n==========\n\nDocumentation should be in UK English.\n\nFollowing the `Python coding guidelines`_ indentation should be 4 spaces\n(no tabs), function and method names should be ``lowercase_with_underscore()``.\nWe\'re using two white lines as separator between methods, as it makes method\nboundries more clear.\n\n\n.. _Python coding guidelines: http://www.python.org/dev/peps/pep-0008/\n.. |Code style: black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :target: https://github.com/ambv/black\n.. |Build status| image:: https://travis-ci.com/gaphor/gaphas.svg?branch=master\n    :target: https://travis-ci.com/gaphor/gaphas\n.. |Coverage| image:: https://coveralls.io/repos/github/gaphor/gaphas/badge.svg?branch=master\n    :target: https://coveralls.io/github/gaphor/gaphas?branch=master\n',
    'author': 'Arjan J. Molenaar',
    'author_email': 'gaphor@gmail.com',
    'url': 'https://gaphas.readthedocs.io/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)



"""
Test all the tools provided by gaphas.
"""

import unittest

from gi.repository import Gtk

from gaphas.canvas import Canvas
from gaphas.canvas import Context
from gaphas.constraint import LineConstraint
from gaphas.examples import Box
from gaphas.item import Line
from gaphas.tool import ConnectHandleTool
from gaphas.view import GtkView

Event = Context


def simple_canvas(self):
    """
    This decorator adds view, canvas and handle connection tool to a test
    case. Two boxes and a line are added to the canvas as well.
    """
    self.canvas = Canvas()

    self.box1 = Box()
    self.canvas.add(self.box1)
    self.box1.matrix.translate(100, 50)
    self.box1.width = 40
    self.box1.height = 40
    self.box1.request_update()

    self.box2 = Box()
    self.canvas.add(self.box2)
    self.box2.matrix.translate(100, 150)
    self.box2.width = 50
    self.box2.height = 50
    self.box2.request_update()

    self.line = Line()
    self.head = self.line.handles()[0]
    self.tail = self.line.handles()[-1]
    self.tail.pos = 100, 100
    self.canvas.add(self.line)

    self.canvas.update_now()
    self.view = GtkView()
    self.view.canvas = self.canvas

    win = Gtk.Window()
    win.add(self.view)
    self.view.show()
    self.view.update()
    win.show()

    self.tool = ConnectHandleTool(self.view)


class ConnectHandleToolGlueTestCase(unittest.TestCase):
    """
    Test handle connection tool glue method.
    """

    def setUp(self):
        simple_canvas(self)

    def test_item_and_port_glue(self):
        """Test glue operation to an item and its ports"""

        ports = self.box1.ports()

        # glue to port nw-ne
        sink = self.tool.glue(self.line, self.head, (120, 50))
        self.assertEqual(sink.item, self.box1)
        self.assertEqual(ports[0], sink.port)

        # glue to port ne-se
        sink = self.tool.glue(self.line, self.head, (140, 70))
        self.assertEqual(sink.item, self.box1)
        self.assertEqual(ports[1], sink.port)

        # glue to port se-sw
        sink = self.tool.glue(self.line, self.head, (120, 90))
        self.assertEqual(sink.item, self.box1)
        self.assertEqual(ports[2], sink.port)

        # glue to port sw-nw
        sink = self.tool.glue(self.line, self.head, (100, 70))
        self.assertEqual(sink.item, self.box1)
        self.assertEqual(ports[3], sink.port)

    def test_failed_glue(self):
        """Test glue from too far distance"""
        sink = self.tool.glue(self.line, self.head, (90, 50))
        self.assertTrue(sink is None)

    #    def test_glue_call_can_glue_once(self):
    #        """Test if glue method calls can glue once only
    #
    #        Box has 4 ports. Every port is examined once per
    #        ConnectHandleTool.glue method call. The purpose of this test is to
    #        assure that ConnectHandleTool.can_glue is called once (for the
    #        found port), it cannot be called four times (once for every port).
    #        """
    #
    #        # count ConnectHandleTool.can_glue calls
    #        class Tool(ConnectHandleTool):
    #            def __init__(self, *args):
    #                super(Tool, self).__init__(*args)
    #                self._calls = 0
    #
    #            def can_glue(self, *args):
    #                self._calls += 1
    #                return True
    #
    #        tool = Tool(self.view)
    #        item, port = tool.glue(self.line, self.head, (120, 50))
    #        assert item and port
    #        self.assertEqual(1, tool._calls)

    #    def test_glue_cannot_glue(self):
    #        """Test if glue method respects ConnectHandleTool.can_glue method"""
    #
    #        class Tool(ConnectHandleTool):
    #            def can_glue(self, *args):
    #                return False
    #
    #        tool = Tool(self.view)
    #        item, port = tool.glue(self.line, self.head, (120, 50))
    #        self.assertTrue(item is None, item)
    #        self.assertTrue(port is None, port)

    def test_glue_no_port_no_can_glue(self):
        """Test if glue method does not call ConnectHandleTool.can_glue method when port is not found"""

        class Tool(ConnectHandleTool):
            def __init__(self, *args):
                super(Tool, self).__init__(*args)
                self._calls = 0

            def can_glue(self, *args):
                self._calls += 1

        tool = Tool(self.view)
        # at 300, 50 there should be no item
        sink = tool.glue(self.line, self.head, (300, 50))
        assert sink is None
        self.assertEqual(0, tool._calls)


class ConnectHandleToolConnectTestCase(unittest.TestCase):
    def setUp(self):
        simple_canvas(self)

    def _get_line(self):
        line = Line()
        head = line.handles()[0]
        self.canvas.add(line)
        return line, head

    def test_connect(self):
        """Test connection to an item"""
        line, head = self._get_line()
        self.tool.connect(line, head, (120, 50))
        cinfo = self.canvas.get_connection(head)
        self.assertTrue(cinfo is not None)
        self.assertEqual(self.box1, cinfo.connected)
        self.assertTrue(cinfo.port is self.box1.ports()[0], "port %s" % cinfo.port)
        self.assertTrue(isinstance(cinfo.constraint, LineConstraint))
        # No default callback defined:
        self.assertTrue(cinfo.callback is None)

        line, head = self._get_line()
        self.tool.connect(line, head, (90, 50))
        cinfo2 = self.canvas.get_connection(head)
        self.assertTrue(cinfo is not cinfo2, cinfo2)
        self.assertTrue(cinfo2 is None, cinfo2)

    def test_reconnect_another(self):
        """Test reconnection to another item"""
        line, head = self._get_line()
        self.tool.connect(line, head, (120, 50))
        cinfo = self.canvas.get_connection(head)
        assert cinfo is not None
        item = cinfo.connected
        port = cinfo.port
        constraint = cinfo.constraint

        assert item == self.box1
        assert port == self.box1.ports()[0]
        assert item != self.box2

        # connect to box2, handle's connected item and connection data
        # should differ
        self.tool.connect(line, head, (120, 150))
        cinfo = self.canvas.get_connection(head)
        assert cinfo is not None
        self.assertEqual(self.box2, cinfo.connected)
        self.assertEqual(self.box2.ports()[0], cinfo.port)

        # old connection does not exist
        self.assertNotEqual(item, cinfo.connected)
        self.assertNotEqual(constraint, cinfo.constraint)

    def test_reconnect_same(self):
        """Test reconnection to same item"""
        line, head = self._get_line()
        self.tool.connect(line, head, (120, 50))
        cinfo = self.canvas.get_connection(head)
        assert cinfo is not None
        item = cinfo.connected
        port = cinfo.port
        constraint = cinfo.constraint

        assert item == self.box1
        assert item != self.box2

        # connect to box1 again, handle's connected item and port should be
        # the same but connection constraint will differ
        connected = self.tool.connect(line, head, (120, 50))
        cinfo = self.canvas.get_connection(head)
        assert cinfo is not None
        self.assertEqual(self.box1, cinfo.connected)
        self.assertEqual(self.box1.ports()[0], cinfo.port)
        self.assertNotEqual(constraint, cinfo.constraint)

    def xtest_find_port(self):
        """Test finding a port
        """
        line, head = self._get_line()
        p1, p2, p3, p4 = self.box1.ports()

        head.pos = 110, 50
        port = self.tool.find_port(line, head, self.box1)
        self.assertEqual(p1, port)

        head.pos = 140, 60
        port = self.tool.find_port(line, head, self.box1)
        self.assertEqual(p2, port)

        head.pos = 110, 95
        port = self.tool.find_port(line, head, self.box1)
        self.assertEqual(p3, port)

        head.pos = 100, 55
        port = self.tool.find_port(line, head, self.box1)
        self.assertEqual(p4, port)


# vim: sw=4:et:ai



"""
Defines aspects for Items. Aspects form intermediate items between
tools and items.

Note: This module uses Phillip J. Eby's simplegeneric module. This
module transforms the generic class (used as fall-back) to a generic
function. In order to inherit from this class you should inherit from
Class.default.  The simplegeneric module is dispatching opnly based on
the first argument.  For Gaphas that's enough.
"""
from __future__ import absolute_import

from builtins import object

from gi.repository import Gdk
from simplegeneric import generic

from gaphas.item import Item, Element


class ItemFinder(object):
    """
    Find an item on the canvas.
    """

    def __init__(self, view):
        self.view = view

    def get_item_at_point(self, pos):
        item, handle = self.view.get_handle_at_point(pos)
        return item or self.view.get_item_at_point(pos)


Finder = generic(ItemFinder)


class ItemSelection(object):
    """
    A role for items. When dealing with selection.

    Behaviour can be overridden by applying the @aspect decorator
    to a subclass.
    """

    def __init__(self, item, view):
        self.item = item
        self.view = view

    def select(self):
        """
        Set selection on the view.
        """
        self.view.focused_item = self.item

    def unselect(self):
        self.view.focused_item = None
        self.view.unselect_item(self.item)


Selection = generic(ItemSelection)


class ItemInMotion(object):
    """
    Aspect for dealing with motion on an item.

    In this case the item is moved.
    """

    def __init__(self, item, view):
        self.item = item
        self.view = view
        self.last_x, self.last_y = None, None

    def start_move(self, pos):
        self.last_x, self.last_y = pos

    def move(self, pos):
        """
        Move the item. x and y are in view coordinates.
        """
        item = self.item
        view = self.view
        v2i = view.get_matrix_v2i(item)

        x, y = pos
        dx, dy = x - self.last_x, y - self.last_y
        dx, dy = v2i.transform_distance(dx, dy)
        self.last_x, self.last_y = x, y

        item.matrix.translate(dx, dy)
        item.canvas.request_matrix_update(item)

    def stop_move(self):
        pass


InMotion = generic(ItemInMotion)


class ItemHandleFinder(object):
    """
    Deals with the task of finding handles.
    """

    def __init__(self, item, view):
        self.item = item
        self.view = view

    def get_handle_at_point(self, pos):
        return self.view.get_handle_at_point(pos)


HandleFinder = generic(ItemHandleFinder)


class ItemHandleSelection(object):
    """
    Deal with selection of the handle.
    """

    def __init__(self, item, handle, view):
        self.item = item
        self.handle = handle
        self.view = view

    def select(self):
        pass

    def unselect(self):
        pass


HandleSelection = generic(ItemHandleSelection)


@HandleSelection.when_type(Element)
class ElementHandleSelection(ItemHandleSelection):
    CURSORS = ("nw-resize", "ne-resize", "se-resize", "sw-resize")

    def select(self):
        index = self.item.handles().index(self.handle)
        if index < 4:
            display = self.view.get_display()
            cursor = Gdk.Cursor.new_from_name(display, self.CURSORS[index])
            self.view.get_window().set_cursor(cursor)

    def unselect(self):
        from .view import DEFAULT_CURSOR

        cursor = Gdk.Cursor(DEFAULT_CURSOR)
        self.view.get_window().set_cursor(cursor)


class ItemHandleInMotion(object):
    """
    Move a handle (role is applied to the handle)
    """

    GLUE_DISTANCE = 10

    def __init__(self, item, handle, view):
        self.item = item
        self.handle = handle
        self.view = view
        self.last_x, self.last_y = None, None

    def start_move(self, pos):
        self.last_x, self.last_y = pos
        canvas = self.item.canvas

        cinfo = canvas.get_connection(self.handle)
        if cinfo:
            canvas.solver.remove_constraint(cinfo.constraint)

    def move(self, pos):
        item = self.item
        handle = self.handle
        view = self.view

        v2i = view.get_matrix_v2i(item)

        x, y = v2i.transform_point(*pos)

        self.handle.pos = (x, y)

        sink = self.glue(pos)

        # do not request matrix update as matrix recalculation will be
        # performed due to item normalization if required
        item.request_update(matrix=False)

        return sink

    def stop_move(self):
        pass

    def glue(self, pos, distance=GLUE_DISTANCE):
        """
        Glue to an item near a specific point.

        Returns a ConnectionSink or None.
        """
        item = self.item
        handle = self.handle
        view = self.view

        if not handle.connectable:
            return None

        connectable, port, glue_pos = view.get_port_at_point(
            pos, distance=distance, exclude=(item,)
        )

        # check if item and found item can be connected on closest port
        if port is not None:
            assert connectable is not None

            connector = Connector(self.item, self.handle)
            sink = ConnectionSink(connectable, port)

            if connector.allow(sink):
                # transform coordinates from view space to the item
                # space and update position of item's handle
                v2i = view.get_matrix_v2i(item).transform_point
                handle.pos = v2i(*glue_pos)
                return sink
        return None


HandleInMotion = generic(ItemHandleInMotion)


class ItemConnector(object):

    GLUE_DISTANCE = 10  # Glue distance in view points

    def __init__(self, item, handle):
        self.item = item
        self.handle = handle

    def allow(self, sink):
        return True

    def glue(self, sink):
        """
        Glue the Connector handle on the sink's port.
        """
        handle = self.handle
        item = self.item
        matrix = item.canvas.get_matrix_i2i(item, sink.item)
        pos = matrix.transform_point(*handle.pos)
        gluepos, dist = sink.port.glue(pos)
        matrix.invert()
        handle.pos = matrix.transform_point(*gluepos)

    def connect(self, sink):
        """
        Connect the handle to a sink (item, port).

        Note that connect() also takes care of disconnecting in case a
        handle is reattached to another element.
        """

        cinfo = self.item.canvas.get_connection(self.handle)

        # Already connected? disconnect first.
        if cinfo:
            self.disconnect()

        if not self.allow(sink):
            return

        self.glue(sink)

        self.connect_handle(sink)

    def connect_handle(self, sink, callback=None):
        """
        Create constraint between handle of a line and port of
        connectable item.

        :Parameters:
         sink
            Connectable item and port.
         callback
            Function to be called on disconnection.
        """
        canvas = self.item.canvas
        handle = self.handle
        item = self.item

        constraint = sink.port.constraint(canvas, item, handle, sink.item)

        canvas.connect_item(
            item, handle, sink.item, sink.port, constraint, callback=callback
        )

    def disconnect(self):
        """
        Disconnect the handle from the attached element.
        """
        self.item.canvas.disconnect_item(self.item, self.handle)


Connector = generic(ItemConnector)


class ItemConnectionSink(object):
    """
    This role should be applied to items that is connected to.
    """

    def __init__(self, item, port):
        self.item = item
        self.port = port

    def find_port(self, pos):
        """
        Glue to the closest item on the canvas.
        If the item can connect, it returns a port.
        """
        port = None
        max_dist = 10e6
        for p in self.item.ports():
            pg, d = p.glue(pos)
            if d >= max_dist:
                continue
            port = p
            max_dist = d

        return port


ConnectionSink = generic(ItemConnectionSink)


##
## Painter aspects
##


class ItemPaintFocused(object):
    """
    Paints on top of all items, just for the focused item and only
    when it's hovered (see gaphas.painter.FocusedItemPainter)
    """

    def __init__(self, item, view):
        self.item = item
        self.view = view

    def paint(self, context):
        pass


PaintFocused = generic(ItemPaintFocused)


# vim:sw=4:et:ai



import unittest
from gaphas.item import Line
from gaphas.canvas import Canvas
from gaphas import state
from gaphas.segment import Segment


undo_list = []
redo_list = []


def undo_handler(event):
    undo_list.append(event)


def undo():
    apply_me = list(undo_list)
    del undo_list[:]
    apply_me.reverse()
    for e in apply_me:
        state.saveapply(*e)
    redo_list[:] = undo_list[:]
    del undo_list[:]


class TestCaseBase(unittest.TestCase):
    """
    Abstract test case class with undo support.
    """

    def setUp(self):
        state.observers.add(state.revert_handler)
        state.subscribers.add(undo_handler)

    def tearDown(self):
        state.observers.remove(state.revert_handler)
        state.subscribers.remove(undo_handler)


class LineTestCase(TestCaseBase):
    """
    Basic line item tests.
    """

    def test_initial_ports(self):
        """Test initial ports amount
        """
        line = Line()
        self.assertEqual(1, len(line.ports()))

    def test_orthogonal_horizontal_undo(self):
        """Test orthogonal line constraints bug (#107)
        """
        canvas = Canvas()
        line = Line()
        canvas.add(line)
        assert not line.horizontal
        assert len(canvas.solver._constraints) == 0

        segment = Segment(line, None)
        segment.split_segment(0)

        line.orthogonal = True

        self.assertEqual(2, len(canvas.solver._constraints))
        after_ortho = set(canvas.solver._constraints)

        del undo_list[:]
        line.horizontal = True

        self.assertEqual(2, len(canvas.solver._constraints))

        undo()

        self.assertFalse(line.horizontal)
        self.assertEqual(2, len(canvas.solver._constraints))

        line.horizontal = True

        self.assertTrue(line.horizontal)
        self.assertEqual(2, len(canvas.solver._constraints))

    def test_orthogonal_line_undo(self):
        """Test orthogonal line undo
        """
        canvas = Canvas()
        line = Line()
        canvas.add(line)

        segment = Segment(line, None)
        segment.split_segment(0)

        # start with no orthogonal constraints
        assert len(canvas.solver._constraints) == 0

        line.orthogonal = True

        # check orthogonal constraints
        self.assertEqual(2, len(canvas.solver._constraints))
        self.assertEqual(3, len(line.handles()))

        undo()

        self.assertFalse(line.orthogonal)
        self.assertEqual(0, len(canvas.solver._constraints))
        self.assertEqual(2, len(line.handles()))


# vim:sw=4:et



import unittest

import cairo

from gaphas.canvas import Canvas, ConnectionError
from gaphas.examples import Box
from gaphas.item import Line


class MatricesTestCase(unittest.TestCase):
    def test_update_matrices(self):
        """Test updating of matrices"""
        c = Canvas()
        i = Box()
        ii = Box()
        c.add(i)
        c.add(ii, i)

        i.matrix = (1.0, 0.0, 0.0, 1.0, 5.0, 0.0)
        ii.matrix = (1.0, 0.0, 0.0, 1.0, 0.0, 8.0)

        updated = c.update_matrices([i])

        self.assertEqual(i._matrix_i2c, cairo.Matrix(1, 0, 0, 1, 5, 0))
        self.assertEqual(ii._matrix_i2c, cairo.Matrix(1, 0, 0, 1, 5, 8))

    def test_reparent(self):
        c = Canvas()
        b1 = Box()
        b2 = Box()
        c.add(b1)
        c.add(b2, b1)
        c.reparent(b2, None)


# fixme: what about multiple constraints for a handle?
#        what about 1d projection?


def count(i):
    return len(list(i))


class CanvasTestCase(unittest.TestCase):
    def test_connect_item(self):
        b1 = Box()
        b2 = Box()
        line = Line()
        c = Canvas()
        c.add(b1)
        c.add(b2)
        c.add(line)

        c.connect_item(line, line.handles()[0], b1, b1.ports()[0])
        assert count(c.get_connections(handle=line.handles()[0])) == 1

        # Add the same
        self.assertRaises(
            ConnectionError, c.connect_item, line, line.handles()[0], b1, b1.ports()[0]
        )
        assert count(c.get_connections(handle=line.handles()[0])) == 1

        # Same item, different port
        # c.connect_item(l, l.handles()[0], b1, b1.ports()[-1])
        # assert count(c.get_connections(handle=l.handles()[0])) == 1

        # Different item
        # c.connect_item(l, l.handles()[0], b2, b2.ports()[0])
        # assert count(c.get_connections(handle=l.handles()[0])) == 1

    def test_disconnect_item_with_callback(self):
        b1 = Box()
        b2 = Box()
        line = Line()
        c = Canvas()
        c.add(b1)
        c.add(b2)
        c.add(line)

        events = []

        def callback():
            events.append("called")

        c.connect_item(line, line.handles()[0], b1, b1.ports()[0], callback=callback)
        assert count(c.get_connections(handle=line.handles()[0])) == 1

        c.disconnect_item(line, line.handles()[0])
        assert count(c.get_connections(handle=line.handles()[0])) == 0
        assert events == ["called"]

    def test_disconnect_item_with_constraint(self):
        b1 = Box()
        b2 = Box()
        line = Line()
        c = Canvas()
        c.add(b1)
        c.add(b2)
        c.add(line)

        cons = b1.ports()[0].constraint(c, line, line.handles()[0], b1)

        c.connect_item(line, line.handles()[0], b1, b1.ports()[0], constraint=cons)
        assert count(c.get_connections(handle=line.handles()[0])) == 1

        ncons = len(c.solver.constraints)
        assert ncons == 5

        c.disconnect_item(line, line.handles()[0])
        assert count(c.get_connections(handle=line.handles()[0])) == 0

        assert len(c.solver.constraints) == 4

    def test_disconnect_item_by_deleting_element(self):
        b1 = Box()
        b2 = Box()
        line = Line()
        c = Canvas()
        c.add(b1)
        c.add(b2)
        c.add(line)

        events = []

        def callback():
            events.append("called")

        c.connect_item(line, line.handles()[0], b1, b1.ports()[0], callback=callback)
        assert count(c.get_connections(handle=line.handles()[0])) == 1

        c.remove(b1)

        assert count(c.get_connections(handle=line.handles()[0])) == 0
        assert events == ["called"]

    def test_disconnect_item_with_constraint_by_deleting_element(self):
        b1 = Box()
        b2 = Box()
        line = Line()
        c = Canvas()
        c.add(b1)
        c.add(b2)
        c.add(line)

        cons = b1.ports()[0].constraint(c, line, line.handles()[0], b1)

        c.connect_item(line, line.handles()[0], b1, b1.ports()[0], constraint=cons)
        assert count(c.get_connections(handle=line.handles()[0])) == 1

        ncons = len(c.solver.constraints)
        assert ncons == 5

        c.remove(b1)

        assert count(c.get_connections(handle=line.handles()[0])) == 0

        self.assertEqual(2, len(c.solver.constraints))


class ConstraintProjectionTestCase(unittest.TestCase):
    def test_line_projection(self):
        """Test projection with line's handle on element's side"""
        line = Line()
        line.matrix.translate(15, 50)
        h1, h2 = line.handles()
        h1.x, h1.y = 0, 0
        h2.x, h2.y = 20, 20

        box = Box()
        box.matrix.translate(10, 10)
        box.width = 40
        box.height = 20
        h_nw, h_ne, h_se, h_sw = box.handles()

        canvas = Canvas()
        canvas.add(line)
        canvas.add(box)

        # move line's second handle on box side
        h2.x, h2.y = 5, -20


#        bc = BalanceConstraint(band=(h_sw.x, h_se.x), v=h2.x, balance=0.25)
#        canvas.projector(bc, x={h_sw.x: box, h_se.x: box, h2.x: line})
#        canvas._solver.add_constraint(bc)
#
#        eq = EqualsConstraint(a=h_se.y, b=h2.y)
#        canvas.projector(eq, y={h_se.y: box, h2.y: line})
#        canvas._solver.add_constraint(eq)
#
#        box.request_update()
#        line.request_update()
#
#        canvas.update()
#
#        box.width = 60
#        box.height = 30
#
#        canvas.update()
#
#        # expect h2.x to be moved due to balance constraint
#        self.assertEqual(10, h2.x)
#        self.assertEqual(-10, h2.y)


class CanvasConstraintTestCase(unittest.TestCase):
    def test_remove_connected_item(self):
        """Test adding canvas constraint"""
        canvas = Canvas()

        from gaphas.aspect import Connector, ConnectionSink

        l1 = Line()
        canvas.add(l1)

        b1 = Box()
        canvas.add(b1)

        number_cons1 = len(canvas.solver.constraints)

        b2 = Box()
        canvas.add(b2)

        number_cons2 = len(canvas.solver.constraints)

        conn = Connector(l1, l1.handles()[0])
        sink = ConnectionSink(b1, b1.ports()[0])

        conn.connect(sink)

        assert canvas.get_connection(l1.handles()[0])

        conn = Connector(l1, l1.handles()[1])
        sink = ConnectionSink(b2, b2.ports()[0])

        conn.connect(sink)

        assert canvas.get_connection(l1.handles()[1])

        self.assertEqual(number_cons2 + 2, len(canvas.solver.constraints))

        canvas.remove(b1)

        # Expecting a class + line connected at one end only
        self.assertEqual(number_cons1 + 1, len(canvas.solver.constraints))


#    def test_adding_constraint(self):
#        """Test adding canvas constraint"""
#        canvas = Canvas()
#
#        l1 = Line()
#        canvas.add(l1)
#
#        h1, h2 = l1.handles()
#        h = Handle()
#
#        eq1 = EqualsConstraint(h1.x, h.x)
#        canvas.add_canvas_constraint(l1, h1, eq1)
#        self.assertTrue(l1 in cons)
#        self.assertTrue(h1 in cons[l1])
#        self.assertTrue(eq1 in cons[l1][h1])
#
#        l2 = Line()
#        canvas.add(l2)
#
#        h1, h2 = l2.handles()
#        h = Handle()
#
#        eq2 = EqualsConstraint(h1.x, h.x)
#        canvas.add_canvas_constraint(l2, h1, eq2)
#        self.assertTrue(l2 in cons)
#        self.assertTrue(h1 in cons[l2])
#        self.assertTrue(eq2 in cons[l2][h1])
#
#
#    def test_adding_constraint_ex(self):
#        """Test adding canvas constraint for non-existing item"""
#        canvas = Canvas()
#        l1 = Line()
#        h1, h2 = l1.handles()
#        h = Handle()
#
#        eq = EqualsConstraint(h1.x, h.x)
#        self.assertRaises(ValueError, canvas.add_canvas_constraint, l1, h1, eq)
#
#
#    def test_removing_constraint(self):
#        """Test removing canvas constraint"""
#        canvas = Canvas()
#        cons = canvas._canvas_constraints
#
#        l1 = Line()
#        canvas.add(l1)
#
#        h1, h2 = l1.handles()
#        h = Handle()
#
#        eq1 = EqualsConstraint(h1.x, h.x)
#        canvas.add_canvas_constraint(l1, h1, eq1)
#
#        # test preconditions
#        assert l1 in cons
#        assert h1 in cons[l1]
#        assert eq1 in cons[l1][h1]
#
#        canvas.remove_canvas_constraint(l1, h1, eq1)
#        self.assertTrue(l1 in cons)
#        self.assertTrue(h1 in cons[l1])
#        self.assertFalse(eq1 in cons[l1][h1])
#
#        eq1 = EqualsConstraint(h1.x, h.x)
#        eq2 = EqualsConstraint(h1.y, h.y)
#        canvas.add_canvas_constraint(l1, h1, eq1)
#        canvas.add_canvas_constraint(l1, h1, eq2)
#
#        # test preconditions
#        assert l1 in cons
#        assert h1 in cons[l1]
#        assert eq1 in cons[l1][h1]
#        assert eq2 in cons[l1][h1]
#
#        canvas.remove_canvas_constraint(l1, h1)
#
#        self.assertTrue(l1 in cons)
#        self.assertTrue(h1 in cons[l1])
#        self.assertFalse(eq1 in cons[l1][h1])
#        self.assertFalse(eq2 in cons[l1][h1])
#
#
#    def test_fetching_constraints(self):
#        """Test fetching canvas constraints"""
#        canvas = Canvas()
#        cons = canvas._canvas_constraints
#
#        l1 = Line()
#        canvas.add(l1)
#
#        h1, h2 = l1.handles()
#        h = Handle()
#
#        eq1 = EqualsConstraint(h1.x, h.x)
#        eq2 = EqualsConstraint(h1.y, h.y)
#        canvas.add_canvas_constraint(l1, h1, eq1)
#        canvas.add_canvas_constraint(l1, h1, eq2)
#
#        # test preconditions
#        assert l1 in cons
#        assert h1 in cons[l1]
#        assert eq1 in cons[l1][h1]
#        assert eq2 in cons[l1][h1]
#
#        self.assertTrue(eq1 in canvas.canvas_constraints(l1))
#        self.assertTrue(eq2 in canvas.canvas_constraints(l1))

# vim:sw=4:et:ai
